#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 22.07.10
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Befehle, die eine Benutzerinteraktion notwendig machen oder
## nicht erwnschte Nebenwirkungen haben, sind auskommentiert.
##
## Um alle Teile des Skripts ausfhren zu knnen, muss das Paket
## mvtnorm
## samt Abhngigkeiten installiert sein, vgl. Abschnitt 1.2.7:
##
## install.packages("mvtnorm", repos="http://ftp5.gwdg.de/pub/misc/cran/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

####*---------------------------------------------------------------------------
##### 2 Elementare Dateneingabe und -verarbeitung
#####---------------------------------------------------------------------------

#####---------------------------------------------------------------------------
##### 2.1 Vektoren
##### 2.1.1
numeric(4)
(age    <- c(18, 20, 30, 24, 23, 21))
addAge  <- c(27, 21, 19)
(ageNew <- c(age, addAge))
length(age)
length("ABCDEF")
nchar("ABCDEF")
nchar(c("A", "BC", "DEF"))

##### 2.1.2
c(11, 12, 13, 14)[2]
age[4]
(ageLast <- age[length(age)])
age[length(age) + 1]

idx <- c(1, 2, 4)
age[idx]
age[c(3, 5, 6)]
age[c(1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6)]
age[c(4, NA, 1)]
age[-3]
age[c(-1, -2, -4)]
age[-c(1, 2, 4)]
age[-idx]

age[4]   <- 25; age
age[idx] <- c(17, 30, 25); age
charVec1 <- c("Z", "Y", "X")
charVec1[c(4, 5, 6)] <- c("W", "V", "U"); charVec1
(charVec2 <- c(charVec1, "T", "S", "R"))
(charVec3 <- append(charVec2, c("Q", "P", "O")))

##### 2.1.3
charVec4 <- "word"
numVec   <- c(10, 20, 30)
(combVec <- c(charVec4, numVec))
mode(combVec)
LETTERS[c(1, 2, 3)]

##### 2.1.4
vec <- c(10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20)
rev(vec)
vec <- c(10, 12, 1, 12, 7, 16, 6, 19, 10, 19)
sort(vec)
(idxDec <- order(vec, decreasing=TRUE))
vec[idxDec]
rank(c(3, 1, 2, 3))
order(c(3, 1, 2, 3))
sort(c("D", "E", "E", "A", "F", "E", "D", "A", "E", "A"))

##### 2.1.5
(namedVec1 <- c(elem1="first", elem2="second"))
namedVec1["elem1"]
(namedVec2 <- c(val1=10, val2=-12, val3=33))
names(namedVec2)
names(namedVec2) <- c("A", "B", "C"); namedVec2

##### 2.1.6
vec <- c(10, 20, 30, 40, 50)
vec <- vec[c(-4, -5)]; vec
vec <- c(1, 2, 3, 4, 5)
length(vec) <- 3; vec

##### 2.1.7
age <- c(18, 20, 30, 24, 23, 21)
age/10
(age/2) + 5
vec1 <- c(3, 4, 5, 6)
vec2 <- c(-2, 2, -2, 2)
vec1*vec2
vec3 <- c(10, 100, 1000, 10000)
(vec1 + vec2) / vec3

##### 2.1.7.1
vec1 <- c(2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24)
vec2 <- c(2, 4, 6, 8, 10)
c(length(age), length(vec1), length(vec2))
vec1*age
vec2*age

##### 2.1.7.2
(zAge <- (age - mean(age)) / sd(age))
(zAge <- scale(age))
as.vector(zAge)
newSd   <- 15
newMean <- 100
(newAge <- (as.vector(zAge)*newSd) + newMean)
mean(newAge)
sd(newAge)

##### 2.1.7.3
height <- c(1.78, 1.91, 1.89, 1.83, 1.64)
weight <- c(65, 89, 91, 75, 73)
(bmi   <- weight / (height^2))

quest1  <- c(FALSE, FALSE, FALSE, TRUE,  FALSE, TRUE, FALSE, TRUE)
quest2  <- c(TRUE,  FALSE, FALSE, FALSE, TRUE,  TRUE, TRUE,  FALSE)
quest3  <- c(TRUE,  TRUE,  TRUE,  TRUE,  FALSE, TRUE, FALSE, FALSE)
(sumVar <- quest1 + quest2 + quest3)

#####---------------------------------------------------------------------------
##### 2.2 Logische Operatoren
##### 2.2.1
age <- c(17, 30, 30, 24, 23, 21)
age < 24
x <- c(2, 4, 8)
y <- c(3, 4, 5)
x == y
x < y
(age <= 20) | (age >= 30)
(age > 20)  & (age < 30)
c(TRUE,  FALSE, FALSE) && c(TRUE,  TRUE, FALSE)
c(FALSE, FALSE, TRUE)  || c(FALSE, TRUE, FALSE)

x <- c(4, 5, 6)
y <- c(4, 5, 6)
z <- c(1, 2, 3)
all.equal(x, y)
all.equal(y, z)
isTRUE(all.equal(y, z))

res <- age > 30
any(res)
any(age < 18)
all(x == y)
res <- age < 24
sum(res)
which(age < 24)
length(which(age < 24))

##### 2.2.2
age[c(TRUE, FALSE, TRUE, TRUE, FALSE, TRUE)]
(idx <- (age <= 20) | (age >= 30))
age[idx]
age[(age >= 30) | (age <= 20)]

vecNA <- c(-3, 2, 0, NA, -7, 5)
(idx  <- vecNA > 0)
vecNA[idx]

(logIdx <- (age < 24))
(numIdx <- which(logIdx))
seq(along=logIdx) %in% numIdx

##### 2.2.3
myColors <- c("red", "blue", "blue", "chartreuse", "red", "green")
farben   <- character(length(myColors))
farben[myColors == "red"]        <- "rot"
farben[myColors == "blue"]       <- "blau"
farben[myColors == "chartreuse"] <- "gelbgrn"
farben[myColors == "green"]      <- "grn"
farben

replace(c(1, 2, 3, 4, 5), list=c(2, 4), values=c(200, 400))

IQ      <- c(112, 103, 87, 86, 90, 101, 90, 89, 122, 103)
IQclass <- numeric(length(IQ))
IQclass[IQ <= 100]                <- 1
IQclass[(IQ > 100) & (IQ <= 115)] <- 2
IQclass[IQ > 115]                 <- 3
IQclass

ifelse(c(6, 2, 4, 5, 4, 2, 5, 4, 6, 6) >= 4, "hi", "lo")

orgVec <- c(5, 9, 11, 8, 9, 3, 1, 13, 9, 12, 5, 12, 6, 3, 17, 5, 8, 7)
cutoff <- 10
(reVec <- ifelse(orgVec <= cutoff, orgVec, cutoff))

#####---------------------------------------------------------------------------
##### 2.3 Mengen
##### 2.3.1
duplicated(c(1, 1, 1, 3, 3, 4, 4))
unique(c(1, 1, 1, 3, 3, 4, 4))
length(unique(c("A", "B", "C", "C", "B", "B", "A", "C", "C", "A")))

##### 2.3.2
x <- c(2, 1, 3, 2, 1)
y <- c(5, 3, 1, 3, 4, 4)
union(x, y)
union(y, x)
intersect(x, y)
intersect(y, x)
setequal(c(1, 1, 2, 2), c(2, 1))
setdiff(x, y)
setdiff(y, x)

is.element(c(29, 23, 30, 17, 30, 10), c(30, 23))
c("A", "Z", "B") %in% c("A", "B", "C", "D", "E")
A <- c(4, 5, 6)
B <- c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
(AinB <- all(A %in% B))
(BinA <- all(B %in% A))
AinB & !BinA

targetSet <- c("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K")
response  <- c("Z", "E", "O", "W", "H", "C", "I", "G", "A", "O", "B")
(respRec  <- ifelse(response %in% targetSet, response, "other"))

##### 2.3.3
myN <- 5
myK <- 4
factorial(myN) / (factorial(myK)*factorial(myN-myK))
choose(myN, myK)

combn(c("a", "b", "c", "d", "e"), myK)
combn(c(1, 2, 3, 4), 3)
combn(c(1, 2, 3, 4), 3, sum)
combn(c(1, 2, 3, 4), 3, weighted.mean, simplify=TRUE, w=c(0.1, 0.2, 0.3))

IV1 <- c("control", "treatment")
IV2 <- c("f", "m")
IV3 <- c(1, 2)
expand.grid(IV1, IV2, IV3)
apply(combn(c(1, 2, 3, 4), 3), 2, sum)

#####---------------------------------------------------------------------------
##### 2.4 Numerische Sequenzen und feste Wertefolgen erzeugen
##### 2.4.1
20:26
26:20
-4:2
-(4:2)
seq(from=2, to=12, by=2)
seq(from=0, to=-1, length.out=5)

age <- c(18, 20, 30, 24, 23, 21)
seq(along=age)
vec <- numeric(0)
length(vec)
1:length(vec)
seq(along=vec)
sequence(c(3, 2))
sequence(1:3)

##### 2.4.2
rep(1:3, times=5)
rep(age, each=2)
vec <- c(1, 4, 8)
(idx <- rep(c(1, 3, 1, 2, 3), times=2))
vec[idx]
rep(c("A", "B", "C"), c(2, 3, 4))

#####---------------------------------------------------------------------------
##### 2.5 Zufallszahlen und zufllige Reihenfolgen generieren
##### 2.5.1
sample(1:6, 20, replace=TRUE)
sample(c("rot", "grn", "blau"), 8, replace=TRUE)

##### 2.5.2
runif(5, min=1, max=6)
rbinom(20, size=5, prob=0.3)
round(rnorm(12, mean=100, sd=15))

##### 2.5.3
vec     <- rep(c("rot", "grn", "blau"), 1000)
selIdx1 <- sample(seq(along=vec), 5, replace=FALSE)
vec[selIdx1]
selIdx2 <- seq(1, length(vec), by=10)
selIdx3 <- rbinom(length(vec), 1, 0.1) == 1

##### 2.5.4
nGroups    <- 3
groupSize  <- 4
(groupsRep <- rep(1:nGroups, groupSize))
(distrib   <- sample(groupsRep, length(groupsRep), replace=FALSE))

myColors  <- c("red", "green", "blue", "yellow", "black")
(randCols <- sample(myColors, length(myColors), replace=FALSE))

#####---------------------------------------------------------------------------
##### 2.6 Deskriptive Kennwerte numerischer Vektoren
summary(c(17, 30, 30, 25, 23, 21))

##### 2.6.1
sum(c(17, 30, 30, 25, 23, 21))
sum(c(17, 30, 30, 25, 23, 21) >= 25)
cumsum(c(17, 30, 30, 25, 23, 21))

diff(c(17, 30, 30, 25, 23, 21))
diff(c(17, 30, 30, 25, 23, 21), lag=2)

prod(c(17, 30, 30, 25, 23, 21))
cumprod(c(17, 30, 30, 25, 23, 21))
factorial(5)

##### 2.6.2
max(c(17, 30, 30, 25, 23, 21))
which.min(c(17, 30, 30, 25, 23, 21))
vec <- c(-5, -8, -2, 10, 9)
val <- 0
which.min(abs(vec-val))
range(c(17, 30, 30, 25, 23, 21))
diff(range(c(17, 30, 30, 25, 23, 21)))
vec1 <- c(5, 2, 0, 7)
vec2 <- c(3, 3, 9, 2)
pmax(vec1, vec2)
pmin(vec1, vec2)

##### 2.6.3
mean(1, 3, 7)
mean(c(1, 3, 7))
mean(c(17, 30, 30, 25, 23, 21))
weights <- c(0.6, 0.6, 0.3, 0.2, 0.4, 0.6)
weighted.mean(c(17, 30, 30, 25, 23, 21), weights)

age <- c(17, 30, 30, 25, 23, 21)
sort(age)
median(age)

ageNew <- c(age, 22)
sort(ageNew)
median(ageNew)

vec     <- c(11, 22, 22, 33, 33, 33, 33)
(tab    <- table(vec))
(modIdx <- which.max(tab))
tab[modIdx]

##### 2.6.4
(quantOld <- quantile(c(17, 30, 30, 25, 23, 21)))
quantOld[c("25%", "50%")]

vec <- sample(seq(0, 1, by=0.01), 1000, replace=TRUE)
quantile(vec, probs=c(0, 0.2, 0.4, 0.6, 0.8, 1))

IQR(c(17, 30, 30, 25, 23, 21))

##### 2.6.5
age <- c(17, 30, 30, 25, 23, 21)
var(age)
sum((age-mean(age))^2) / (length(age)-1)

((length(age)-1) / length(age)) * var(age)
sum((age-mean(age))^2) / length(age)

sqrt(var(age))
sd(age)
sqrt((length(age)-1) / length(age)) * sd(age)
sqrt(sum((age-mean(age))^2) / length(age))

mean(abs(age-median(age)))
mad(age)

##### 2.6.6
age      <- c(17, 30, 30, 25, 23, 21)
semester <- c(1, 12, 8, 10, 5, 3)
cov(age, semester)
sum((age-mean(age)) * (semester-mean(semester))) / (length(age)-1)
((length(age)-1) / length(age)) * cov(age, semester)
sum((age-mean(age)) * (semester-mean(semester))) / length(age)
cor(age, semester)
cov(age, semester) / (sd(age) * sd(semester))

nObs <- 100
z    <- runif(nObs)
x    <- z + rnorm(nObs, 0, 0.5)
y    <- z + rnorm(nObs, 0, 0.5)
## fr Daten des Buches diese Datei laden:
# load("data_regrParCor.Rdata")
(cor(x, y)-(cor(x, z) * cor(y, z))) / (sqrt((1-cor(x, z)^2) * (1-cor(y, z)^2)))

##### 2.6.7
outer(1:10, 1:10, "*")

#####---------------------------------------------------------------------------
##### 2.7 Gruppierungsfaktoren
##### 2.7.1
sex     <- c("m", "f", "f", "m", "m", "m", "f", "m", "f", "f")
(sexFac <- factor(sex))
factor(c(1, 1, 3, 3, 4, 4), levels=1:5)
(sexNum <- rbinom(10, 1, 0.5))
factor(sexNum, labels=c("man", "woman"))
levels(sexFac) <- c("fem", "male"); sexFac

nlevels(sexFac)
summary(sexFac)
levels(sexFac)
str(sexFac)
unclass(sexFac)
unclass(factor(10:15))
as.character(sexFac)

##### 2.7.2
(status   <- factor(c("hi", "lo", "hi")))
status[4] <- "mid"; status
levels(status) <- c(levels(status), "mid")
status[4] <- "mid"; status

hiNotHi <- status
levels(hiNotHi) <- list(hi="hi", notHi=c("mid", "lo")); hiNotHi

status[1:2]
status[1:2, drop=TRUE]
(newStatus <- factor(status[1:2]))

##### 2.7.3
ordered(status, levels=c("lo", "mid", "hi"))

##### 2.7.4
fac <- factor(rep(LETTERS[1:3], each=10))
vec <- rnorm(30, rep(c(10, 5, 15), each=10), 3)
tapply(vec, fac, mean)
reorder(fac, vec, mean)

(fac <- factor(sample(1:2, 10, replace=TRUE), labels=c("B", "A")))
sort(fac)
as.vector(fac)
sort(as.vector(fac))

##### 2.7.5
(myFac1 <- factor(rep(c("A", "B"), c(5, 5))))
(myFac2 <- gl(2, 5, labels=c("less", "more"), ordered=TRUE))
sample(myFac2, length(myFac2), replace=FALSE)

expand.grid(IV1=gl(2, 2, labels=c("a", "b")), IV2=gl(3, 1))

##### 2.7.6
IQ    <- rnorm(100, mean=100, sd=15)
IQfac <- cut(IQ, breaks=c(0, 85, 115, Inf)); IQfac[1:5]
levels(IQfac) <- c("lo", "mid", "hi"); IQfac[1:5]

##### 2.7.7
nSubj <- 2
P     <- 2
Q     <- 3
sex   <- factor(rep(c("f", "m"), Q*nSubj))
group <- factor(rep(c("T", "WL", "CG"), each=P*nSubj))
table(sex, group)

IQ <- round(rnorm(P*Q*nSubj, mean=100, sd=15))
ave(IQ, sex, FUN=mean)

(tapRes <- tapply(IQ, group, FUN=mean))
tapply(IQ, list(sex, group), FUN=mean)
tapRes[group]

tapply(IQ, sex, "[")
IQ[sex == "f"]
IQ[sex == "m"]

#####---------------------------------------------------------------------------
##### 2.8 Matrizen
age <- c(17, 30, 30, 25, 23, 21)
matrix(age, nrow=3, ncol=2, byrow=FALSE)
(ageMat <- matrix(age, nrow=2, ncol=3, byrow=TRUE))

##### 2.8.1
##### 2.8.2
age    <- c(17, 30, 30, 25, 23, 21)
ageMat <- matrix(age, nrow=2, ncol=3, byrow=FALSE)
dim(ageMat)
nrow(ageMat)
ncol(ageMat)
prod(dim(ageMat))
dim(age) <- c(2, 3)
age

as.matrix(1:4)
c(ageMat)

P        <- 2
Q        <- 3
(pqMat   <- matrix(1:(P*Q), nrow=P, ncol=Q))
(rowMat1 <- row(pqMat))
(colMat1 <- col(pqMat))

rowMat2  <- matrix(rep(1:P, Q),      ncol=Q)
colMat2  <- matrix(rep(1:Q, each=P), ncol=Q)

cbind(rowIdx=c(rowMat1), colIdx=c(colMat1), val=c(pqMat))

mat <- matrix(sample(1:10, 16, replace=TRUE), 4, 4)
col(mat) >= row(mat)

##### 2.8.3
ageMat[2, 2]
ageMat[2, 2] <- 24; ageMat[2, 2]
ageMat[2,  ]
ageMat[ , 1]
ageMat[ ,  ]
ageMat[ , 1, drop=FALSE]
ageMat[ , 2:3]
ageMat[ , c(1, 3)]

ageMatNew   <- ageMat
(replaceMat <- matrix(c(11, 21, 12, 22), 2, 2))
ageMatNew[ , c(1, 3)] <- replaceMat
ageMatNew

##### 2.8.4
ageMat[c(1, 3, 4)]
(idxMat <- matrix(c(1, 1, 2, 2, 3, 3), 3, 2))
ageMat[idxMat]

(selMat <- ageMat >= 25)
ageMat[selMat]

##### 2.8.5
nSubj  <- 10
age    <- sample( 18:35,  nSubj, replace=TRUE)
weight <- sample( 65:95,  nSubj, replace=TRUE)
height <- sample(168:201, nSubj, replace=TRUE)
rbind(age, weight, height)
varMat <- cbind(age, weight, height); varMat[1:5, ]

##### 2.8.6
ageMat
sum(ageMat)
rowSums(ageMat)
mean(ageMat)
colMeans(ageMat)

##### 2.8.7
apply(ageMat, 2, sum)
apply(ageMat, 1, max)
apply(ageMat, 2, range)
mat <- matrix(sample(1:100, 120, replace=TRUE), ncol=3)
apply(mat, 2, mean, trim=0.1)

##### 2.8.8
rowMs <- rowMeans(ageMat)
colMs <- colMeans(ageMat)
sweep(ageMat, 1, rowMs, "-")
sweep(ageMat, 2, colMs, "-")
scale(ageMat, center=TRUE, scale=FALSE)
ageMat - matrix(rep(colMs, nrow(ageMat)), nrow=nrow(ageMat), byrow=TRUE)
n <- nrow(ageMat)
(diag(n) - (1/n) * (rep(1, n) %*% t(rep(1, n)))) %*% ageMat

##### 2.8.9
cov(varMat)
cor(varMat)
cov.wt(varMat, method="ML")
diag(cov(varMat))

vec <- rnorm(nSubj)
cor(varMat, vec)

##### 2.8.10
nSubj  <- 5
sex    <- sample(c(0, 1), nSubj, replace=TRUE)
age    <- sample(  18:35, nSubj, replace=TRUE)
rating <- sample(    1:3, nSubj, replace=TRUE)
(mat   <- cbind(sex, age, rating))

(rowOrder1 <- order(mat[ , 1]))
mat[rowOrder1, ]

rowOrder2 <- order(mat[ , 1], partial=mat[ , 2])
mat[rowOrder2, ]

order(mat[ , 1], mat[ , 2])

rowOrder3 <- order(mat[ , 1], -mat[ , 2])
mat[rowOrder3, ]

#####---------------------------------------------------------------------------
##### 2.9 Lineare Algebra
##### 2.9.1
nn <- 4
qq <- 2
(X <- matrix(c(20, 26, 10, 19, 29, 27, 20, 12), nrow=nn, ncol=qq))
t(X)

diag(cov(X))
diag(1:3)
diag(2)

(Xc <- diag(nn) - (1/nn) * (rep(1, nn) %*% t(rep(1, nn))))
## Alternative:
# (Xc <- diag(nn) - matrix(rep(1/nn, nn^2), nrow=nn))
# all(Xc %*% Xc == Xc)
## besserer Weg aufgrund potentieller Rundungsfehler:
all.equal(Xc %*% Xc, Xc)
(Xdot <- Xc %*% X)
colSums(Xdot)
(SSCP <- t(Xdot) %*% Xdot)

(1/(nn-1)) * SSCP
cov(X)

(1:nn)     %*% t(rep(1, qq))
rep(1, nn) %*% t(1:qq)

##### 2.9.2
mat     <- matrix(c(1, 1, 1, -1), nrow=2)
(matInv <- solve(mat))
mat %*% matInv

A  <- matrix(c(9, 1, -5, 0), nrow=2)
b  <- c(5, -3)
(x <- solve(A, b))
A %*% x

##### 2.9.3
vec1 <- c(3, 4, 1, 8, 2)
sqrt(crossprod(vec1))
sqrt(sum(vec1^2))

vec2 <- c(6, 9, 10, 8, 7)
mat1 <- cbind(vec1, vec2)
sqrt(diag(crossprod(mat1)))
sqrt(colSums(mat1^2))

mat2 <- matrix(sample(-20:20, 12, replace=TRUE), ncol=3)
dist(mat2, diag=TRUE, upper=TRUE)
sqrt(crossprod(mat2[1, ] - mat2[2, ]))

sigma <- matrix(c(4,2,-3, 2,16,-1, -3,-1,9), byrow=TRUE, ncol=3)
mu    <- c(-3, 2, 4)
library(mvtnorm)
nSubj <- 100
X     <- round(rmvnorm(n=nSubj, mean=mu, sigma=sigma))
## fr Daten des Buches diese Datei laden:
# load("data_mahaDist.Rdata")
ctr   <- colMeans(X)
S     <- cov(X)

Seig  <- eigen(S)
sqrtD <- sqrt(Seig$values)
SsqrtInv <- Seig$vectors %*% diag(1/sqrtD) %*% t(Seig$vectors)

Xctr  <- sweep(X, 2, ctr, "-")
Xm    <- t(SsqrtInv %*% t(Xctr))
cov(Xm)
colMeans(Xm)

ideal <- c(1, 2, 3)
x     <- X[1, ]
y     <- X[2, ]
mat   <- rbind(x, y)

mahalanobis(mat, ideal, S)

Sinv <- solve(S)
t(x-ideal) %*% Sinv %*% (x-ideal)
t(y-ideal) %*% Sinv %*% (y-ideal)
matDiff <- sweep(mat, 2, ideal, "-")
diag(matDiff %*% Sinv %*% t(matDiff))

mDist   <- mahalanobis(X, ideal, S)
min(mDist)
(idxMin <- which.min(mDist))
X[idxMin, ]

idealM <- t(SsqrtInv %*% (ideal - ctr))
crossprod(Xm[1, ] - t(idealM))
crossprod(Xm[2, ] - t(idealM))

matM     <- rbind(Xm[1, ], Xm[2, ])
matMdiff <- sweep(matM, 2, idealM, "-")
diag(crossprod(t(matMdiff)))

##### 2.9.4
## fr Daten des Buches diese Datei laden:
# load("data_mahaDist.Rdata")
ones <- rep(1, nrow(X))
P1   <- ones %*% solve(t(ones) %*% ones) %*% t(ones)
Px1  <- P1 %*% X
Px1[1:3, ]

colMeans(X)

a  <- ones / sqrt(crossprod(ones))
P2 <- a %*% t(a)
all.equal(P1, P2)

A   <- cbind(c(1, 0, 0), c(0, 1, 0))
P3  <- A %*% solve(t(A) %*% A) %*% t(A)
Px3 <- t(P3 %*% t(X))
Px3[1:3, ]
X[1:3, ]

##### 2.9.5
nn <- 4
qq <- 2
X  <- matrix(c(20, 26, 10, 19, 29, 27, 20, 12), nrow=nn, ncol=qq)
Xc <- diag(nn) - (1/nn) * (rep(1, nn) %*% t(rep(1, nn)))
sum(diag(Xc))
det(Xc)

mat    <- matrix(c(1, 1, 1, -1), nrow=2)
matInv <- solve(mat)
all.equal(det(matInv), 1/det(mat))

qrRes <- qr(Xc)
qrRes$rank
qr.Q(qrRes) %*% qr.R(qrRes)

mat  <- matrix(c(1, 0, 0, 2), nrow=2)
(eig <- eigen(mat))
eig$vectors %*% t(eig$vectors)
all.equal(sum(diag(mat)),  sum(eig$values))
all.equal(det(mat),       prod(eig$values))

eig$vectors %*% diag(eig$values) %*% t(eig$vectors)

sqrtD <- sqrt(eig$values)
AA    <- eig$vectors %*% diag(sqrtD) %*% t(eig$vectors)
AA %*% AA

AAinv <- eig$vectors %*% diag(1/sqrtD) %*% t(eig$vectors)
AAinv %*% AA

N <- eig$vectors %*% sqrt(diag(eig$values))
N %*% t(N)

BB <- eig$vectors %*% diag(c(eig$values[1], 0)) %*% t(eig$vectors)
CC <- eig$vectors %*% diag(c(0, eig$values[2])) %*% t(eig$vectors)
BB + CC

X <- matrix(c(20, 26, 10, 19, 29, 27, 20, 12), nrow=4, ncol=2)
kappa(X, exact=TRUE)
eigVals <- eigen(t(X) %*% X)$values
sqrt(max(eigVals) / min(eigVals))

Xsvd <- svd(X)
Xsvd$u %*% diag(Xsvd$d) %*% Xsvd$v

R <- chol(cov(X))
t(R) %*% R
cov(X)

#####---------------------------------------------------------------------------
##### 2.10 Arrays
(myArr1 <- array(1:12, c(2, 3, 2),
                 dimnames=list(c("f", "m"), c("CG", "WL", "T"), c("high", "low"))))
myArr1[1, 3, 2]
myArr2 <- myArr1*2
myArr2[ , , 1]

#####---------------------------------------------------------------------------
##### 2.11 Hufigkeitsauszhlungen
##### 2.11.1
(myLetters <- sample(LETTERS[1:5], 12, replace=TRUE))
## fr Daten des Buches diese Datei laden:
# load("data_myLetters.Rdata")
(tab       <- table(myLetters))
names(tab)
tab["B"]
names(tab)[1] <- "F"; tab

table(myLetters) / length(myLetters)
prop.table(table(myLetters))

tab["Q"] <- 0; tab
tabOrder <- order(names(tab))
tab[tabOrder]

letFac         <- factor(myLetters, levels=LETTERS[1:5])
levels(letFac) <- c(levels(letFac), "Q"); letFac
table(letFac)

##### 2.11.2
tabulate(c(1, 2, 2, 3, 3, 3))
tabulate(c(1, 2, 2, 4, 4, 4, 4), nbins=5)
tabulate(c(-1, 0, 1, 2, 2, 4, 4, 4, 4, 6), nbins=5)

##### 2.11.3
(vec <- rep(rep(c("f", "m"), 3), c(1, 3, 2, 4, 1, 2)))
(res <- rle(vec))
length(res$lengths)
inverse.rle(res)

##### 2.11.4
(work    <- factor(sample(c("home", "office"), 10, replace=TRUE)))
(sex     <- factor(sample(c("f", "m"), 10, replace=TRUE)))
## fr Daten des Buches diese Datei laden:
# load("data_xtabs.Rdata")
table(sex, work)
(relFreq <- prop.table(table(sex, work)))
rSums    <- rowSums(relFreq)
cSums    <- colSums(relFreq)
sweep(relFreq, 1, rSums, "/")
sweep(relFreq, 2, cSums, "/")
## einfacherer Weg ber das margin=<Zahl> Argument von prop.table()
# prop.table(table(sex, work), margin=1)
# prop.table(table(sex, work), margin=2)

(group <- factor(sample(c("A", "B"), 10, replace=TRUE)))
ftable(work, sex, group, row.vars="work", col.vars=c("sex", "group"))
(persons <- data.frame(sex, work))
xtabs(~ sex + work, data=persons)
summary(table(sex, work))

##### 2.11.5
## fr Daten des Buches diese Datei laden:
# load("data_xtabs.Rdata")
apply(xtabs(data=persons), MARGIN=1, FUN=sum)
colMeans(xtabs(data=persons))
addmargins(xtabs(data=persons), c(1, 2), mean)

##### 2.11.6
(vec    <- round(rnorm(10), 2))
(myStep <- ecdf(vec))
myStep(vec)
myStep(sort(vec))
knots(myStep)

steps   <- 1:3
heights <- c(1, 2, 4, 3)
(sFun0  <- stepfun(steps, heights, f=0))
sFun25  <- stepfun(steps, heights, f=0.25)
sFun50  <- stepfun(steps, heights, f=0.50)
sFun75  <- stepfun(steps, heights, f=0.75)
sFun1   <- stepfun(steps, heights, f=1)
x       <- seq(from=0.5, to=3.5, by=0.5)
cbind(x, f0=sFun0(x), f25=sFun25(x), f50=sFun50(x), f75=sFun75(x), f1=sFun1(x))

dev.new(width=12, height=4.5)
op <- par(mfrow=c(1, 3), cex.lab=1.5, cex.main=1.5)
plot(sFun0)
plot(sFun50)
plot(sFun1)
par(op)

#####---------------------------------------------------------------------------
##### 2.12 Codierung, Identifikation und Behandlung fehlender Werte
##### 2.12.1
(vec1 <- c(10, 20, NA, 40, 50, NA))
length(vec1)
is.na(vec1)

vec2   <- c(NA, 7, 9, 10, 1, 8)
(matNA <- rbind(vec1, vec2))
is.na(matNA)

any(is.na(vec1))
sum(is.na(vec1))
which(is.na(vec1))

##### 2.12.2
vecOrg     <- c(30, 25, 23, 21, 99, 999)
vecNA      <- vecOrg
(idx       <- (vecOrg == 99) | (vecOrg == 999))
idx        <- vecOrg %in% c(99, 999)
vecNA[idx] <- NA; vecNA

(matOrg    <- matrix(vecOrg, 2, 3))
matNA      <- matOrg
idx        <- (matOrg == 99) | (matOrg == 999)
matNA[idx] <- NA; matNA

##### 2.12.3
goodIdx <- !is.na(vecNA)
mean(vecNA[goodIdx])
sd(na.omit(vecNA))

sum(vecNA)
sum(vecNA, na.rm=TRUE)
apply(matNA, 1, mean)
apply(matNA, 1, mean, na.rm=TRUE)

##### 2.12.4
##### 2.12.4.1
ageNA  <- c(18, NA, 27, 22)
DV1    <- c(NA, 1, 5, -3)
DV2    <- c(9, 4, 2, 7)
(matNA <- cbind(ageNA, DV1, DV2))

(rowNAidx <- apply(is.na(matNA), 1, any))
matNA[!rowNAidx, ]

na.omit(matNA)
colMeans(na.omit(matNA))

cov(matNA, use="complete.obs")
all(cov(matNA, use="complete.obs") == cov(na.omit(matNA)))

##### 2.12.4.2
rowMeans(matNA)
rowMeans(mat, na.rm=TRUE)
cov(matNA, use="pairwise.complete.obs")

##### 2.12.5

#####---------------------------------------------------------------------------
##### 2.13 Zeichenketten verarbeiten
##### 2.13.1
randVals <- round(rnorm(5), 2)
toString(randVals)
formatC(c(1, 2.345), width=5, format="f")

##### 2.13.2
paste("group", LETTERS[1:5], sep="_")
paste(1:5, palette()[1:5], sep=": ")

nSubj <- 20
gName <- "A"
mVal  <- 14.2
sprintf("For the %d particpants in group %s, the mean was %f", nSubj, gName, mVal)
sprintf("%.3f", 1.23456)

cVar <- "A string"
cat(cVar, "with\n", 4, "\nwords\n", sep="+")

##### 2.13.3
tolower(c("A", "BC", "DEF"))
toupper(c("ghi", "jk", "i"))
strReverse <- function(x) { sapply(lapply(strsplit(x, NULL), rev), paste, collapse="") }
strReverse(c("Lorem", "ipsum", "dolor", "sit"))
substring(c("ABCDEF", "GHIJK", "LMNO", "PQR"), 4, 5)

charVec <- c("ABCDEF", "GHIJK", "LMNO", "PQR")
substring(charVec, 4, 5) <- c("..", "xx", "++", "**"); charVec

strsplit(c("abc_def_ghi", "jkl_mno"), "_")

##### 2.13.4
 match(c("abc", "de", "f", "h"), c("abcde", "abc", "de", "fg", "ih"))
pmatch(c("abc", "de", "f", "h"), c("abcde", "abc", "de", "fg", "ih"))
grep( "A[BC][[:blank:]]", c("AB ", "AB", "AC ", "A "))
grepl("A[BC][[:blank:]]", c("AB ", "AB", "AC ", "A "))
glob2rx("asdf*.txt")

##### 2.13.5
sub("em", "XX", "Lorem ipsum dolor sit Lorem ipsum")
gsub("em", "XX", "Lorem ipsum dolor sit Lorem ipsum")

##### 2.13.6
obj1 <- parse(text="3 + 4")
obj2 <- parse(text=c("vec <- c(1, 2, 3)", "vec^2"))
eval(obj1)
eval(obj2)

#####---------------------------------------------------------------------------
##### 2.14 Datum und Uhrzeit
##### 2.14.1
Sys.Date()

(myDate <- as.Date("01.11.1974", format="%d.%m.%Y"))
format(myDate, format="%d.%m.%Y")
myDate + 365

(diffDate <- as.Date("1976-06-19") - myDate)
myDate + diffDate

(myTime   <- as.POSIXct("2009-02-07 09:23:02"))
charDates <- c("05.08.1972, 03:37", "31.03.1981, 12:44")
(lDates   <- strptime(charDates, format="%d.%m.%Y, %H:%M"))
lDates$mday

lDates + 120
(diff21 <- lDates[2] - lDates[1])
lDates[1] + diff21

##### 2.14.2

