#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 22.07.10
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Befehle, die eine Benutzerinteraktion notwendig machen oder
## nicht erwnschte Nebenwirkungen haben, sind auskommentiert.
##
## Um alle Teile des Skripts ausfhren zu knnen, mssen die Pakete
## car, scatterplot3d
## samt Abhngigkeiten installiert sein, vgl. Abschnitt 1.2.7:
##
## install.packages(c("car", "scatterplot3d"), repos="http://ftp5.gwdg.de/pub/misc/cran/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

####*---------------------------------------------------------------------------
##### 7 Korrelation und Regressionsanalyse
#####---------------------------------------------------------------------------

#####---------------------------------------------------------------------------
##### 7.1 Test auf Korrelation
nSubj <- 20
vec1  <- sample(0:100, nSubj, replace=TRUE)
vec2  <- vec1 + rnorm(nSubj, 0, 50)
## fr Daten des Buches diese Datei laden:
# load("data_corTest.Rdata")
cor.test(vec1, vec2)

r      <- cor(vec1, vec2)
(tVal  <- sqrt(nSubj-2) * r / sqrt(1-r^2))
(pVal  <- 2*(1-pt(tVal, nSubj-2)))

fishZ  <- 0.5 * log((1+r)/(1-r))
fishV  <- 1 / (nSubj-3)
zCrit  <- qnorm(1-(0.05/2), 0, 1)
zLo    <- fishZ - zCrit*sqrt(fishV)
zUp    <- fishZ + zCrit*sqrt(fishV)
(ciLo  <- (exp(1)^(2*zLo)-1) / (exp(1)^(2*zLo)+1))
(ciUp  <- (exp(1)^(2*zUp)-1) / (exp(1)^(2*zUp)+1))

#####---------------------------------------------------------------------------
##### 7.2 Einfache lineare Regression
nSubj  <- 100
height <- rnorm(nSubj, 175, 7)
weight <- 0.4*height + 10 + rnorm(nSubj, 0, 3)
## fr Daten des Buches diese Datei laden:
# load("data_linRegr.Rdata")
(model <- lm(weight ~ height))

(b <- cov(weight, height) / var(height))
(a <- mean(weight) - b*mean(height))
fitVals <- b * height + a

lm(scale(weight) ~ scale(height))
b * sd(height) / sd(weight)
cor(height, weight)

names(model)

dev.new(width=7, height=7)
plot(height, weight, xlab="height [cm]", ylab="weight [kg]", pch=20,
     main="Daten mit Zentroid, Regressionsgerade und Modellgerade", cex.lab=1.4)
abline(model, col="blue", lwd=2)
abline(a=10, b=0.4, col="gray", lwd=2)
points(mean(height), mean(weight), col="red", pch=4, lwd=3, cex=1.5)
legend(x="topleft", legend=c("Daten", "Zentroid", "Regressionsgerade", "Modellgerade"),
       col=c("black", "red", "blue", "gray"), lwd=c(1, 3, 2, 2),
       pch=c(20, 4, NA, NA), lty=c(NA, NA, 1, 1))

##### 7.2.1
## fr Daten des Buches diese Datei laden:
# load("data_linRegr.Rdata")
(sumRes <- summary(model))
## options(show.signif.stars=FALSE)

P      <- 1
(dfSSE <- nSubj-P-1)
MSE    <- sum(residuals(model)^2) / dfSSE
sqrt(MSE)

(rSq    <- sumRes$r.squared)
fitVals <- fitted(model)
cor(fitVals, weight)^2
1 - ((nSubj-1)/(nSubj-P-1)) * (1-rSq)

SScrit   <- sum((weight-mean(weight))^2)
dfSScrit <- nSubj-1
1 - (MSE / (SScrit / dfSScrit))

MSpred <- sum((fitVals-mean(fitVals))^2) / P
(Fval  <- MSpred / MSE)
(pVal  <- 1-pf(Fval, P, dfSSE))

##### 7.2.2
dev.new(width=12, height=4.5)
op <- par(mfrow=c(1, 3), cex.lab=1.5, cex.main=1.5)
qqnorm(residuals(model))
qqline(residuals(model), col="red", lwd=2)
plot(fitted(model), residuals(model), pch=20, xlab="Vorhersage", ylab="Residuen",
     main="Residuen gegen Vorhersage")
abline(h=0, col="blue", lwd=2)
termplot(model, partial.resid=TRUE, rug=TRUE, xlab="Prdiktor", ylab="Residuen",
         main="Residuen, Prdiktoren, Regr.gerade")
mtext(text="Einzelwerte Prdiktor", side=1)
mtext(text="Einzelwerte Kriterium", side=2)
par(op)

##### 7.2.3
myData   <- round(rnorm(100, 75, 10))
crit     <- (0.5*myData + 133) + rnorm(100, 0, 10)
## fr Daten des Buches diese Datei laden:
# load("data_linRegrVI.Rdata")
model    <- lm(crit ~ myData)
(coeffs  <- coefficients(model))

newData  <- sample(50:150, 5, replace=TRUE)
(newPred <- coeffs[2]*newData + coeffs[1])

newDf    <- data.frame(myData=newData)
predict(model, newDf, interval="confidence", level=0.95)

orgPred  <- predict(model, interval="confidence", level=0.95)
dev.new(width=7, height=7)
op <- par(lend=2, cex.lab=1.4)
plot(myData, crit, pch=20, xlab="Prdiktor", ylab="Kriterium und Vorhersage",
     main="Daten und Vorhersage durch Regression")
myOrder <- order(myData)
polygon(c(myData[myOrder], rev(myData[myOrder])),
        c(orgPred[myOrder, "lwr"], orgPred[rev(myOrder), "upr"]),
        border=NA, col=rgb(0.7, 0.7, 0.7, 0.6))
abline(model, col="blue")
legend(x="bottomright", legend=c("Daten", "Vorhersage", "Vertrauensbereich"),
       pch=c(20, NA, NA), lty=c(NA, 1, 1), lwd=c(NA, 1, 8),
       col=c("black", "blue", "gray"))
par(op)

##### 7.2.4
z <- runif(100)
x <- z + rnorm(100, 0, 0.5)
y <- z + rnorm(100, 0, 0.5)
## fr Daten des Buches diese Datei laden:
# load("data_regrParCor.Rdata")
cor(cbind(z, x, y))

model1 <- lm(x ~ z)
model2 <- lm(y ~ z)
cor(residuals(model1), residuals(model2))

(cor(x,y) - (cor(x,z) * cor(y,z))) / (sqrt((1-cor(x,z)^2) * (1-cor(y,z)^2)))

##### 7.2.5
nSubj    <- 100
predAll  <- sample(50:150, nSubj, replace=TRUE)
critAll  <- predAll + rnorm(nSubj, mean=0, sd=30)

select   <- sample(rep(c(TRUE, FALSE), nSubj/2), nSubj, replace=FALSE)
## fr Daten des Buches diese Datei laden:
# load("data_regrCrVal.Rdata")
pred1    <- predAll[ select]
crit1    <- critAll[ select]
pred2    <- predAll[!select]
crit2    <- critAll[!select]

model1   <- lm(crit1 ~ pred1)
model2   <- lm(crit2 ~ pred2)
(coeffs1 <- coefficients(model1))
(coeffs2 <- coefficients(model2))

pred1in2 <- pred2*coeffs1[2] + coeffs1[1]
pred2in1 <- pred1*coeffs2[2] + coeffs2[1]

(cor(pred1in2, crit2))^2
(cor(pred2in1, crit1))^2

#####---------------------------------------------------------------------------
##### 7.3 Multiple lineare Regression
nSubj   <- 100
height  <- rnorm(nSubj, 175, 7)
age     <- rnorm(nSubj, 30, 8)
sport   <- abs(rnorm(nSubj, 60, 30))
weight  <- 0.5*height - 0.3*age - 0.4*sport + 10 + rnorm(nSubj, 0, 3)
## fr Daten des Buches diese Datei laden:
# load("data_multRegr.Rdata")
(model1 <- lm(weight ~ height + age))

X  <- cbind(height, age)
K  <- cov(X)
k  <- cov(X, weight)
(b <- solve(K, k))
(a <- as.vector(mean(weight) - t(b) %*% colMeans(X)))
fitVals <- X %*% b + a

lm(scale(weight) ~ scale(height) + scale(age))
(1/sd(weight)) * diag(sqrt(diag(K))) %*% b

b0 <- model1$coefficients[1]
b1 <- model1$coefficients[2]
b2 <- model1$coefficients[3]

dev.new(width=7, height=7)
library(scatterplot3d)
sp3 <- scatterplot3d(height, age, weight, pch=16, color="red",
                     zlab="Kriterium: weight", mar=c(3, 3, 4, 2),
                     main="Multiple Regression: Daten und Vorhersageebene",
                     cex.main=1.4, cex.lab=1.4)
sp3$plane3d(model1)
xyEmp  <- sp3$xyz.convert(height, age, weight)
xyPred <- sp3$xyz.convert(height, age, b1*height + b2*age + b0)
segments(xyPred$x, xyPred$y, xyEmp$x, xyEmp$y, col="blue")

##### 7.3.1
## fr Daten des Buches diese Datei laden:
# load("data_multRegr.Rdata")
(model2 <- update(model1, . ~ . + sport))
(model3 <- update(model2, . ~ . - age))

##### 7.3.2
## fr Daten des Buches diese Datei laden:
# load("data_multRegr.Rdata")
add1(model1, . ~ . + sport)

##### 7.3.3
## fr Daten des Buches diese Datei laden:
# load("data_multRegr.Rdata")
X3 <- cbind(height, age, sport)
cor(X3)

library(car)
vif(model2)

kappa(lm(scale(weight) ~ scale(height) + scale(age) + scale(sport)), exact=TRUE)

#####---------------------------------------------------------------------------
##### 7.4 Logistische Regression
nSubj   <- 100
height  <- rnorm(nSubj, 175, 7)
weight  <- 0.4*height + 10 + rnorm(nSubj, 0, 3)
wFac    <- cut(weight, breaks=c(-Inf, median(weight), Inf))
levels(wFac) <- c("lo", "hi")
regDf   <- data.frame(wFac, height)
## fr Daten des Buches diese Datei laden:
# load("data_logRegr.Rdata")
(glmRes <- glm(wFac ~ height, family=binomial(link="logit"), data=regDf))

exp(glmRes$coefficients)

logLik(glmRes)
fitted(glmRes)

b0     <- glmRes$coefficients[1]
b1     <- glmRes$coefficients[2]
thresh <- -b0/b1
wrong  <- (height < thresh & wFac == "hi") | (height >= thresh & wFac == "lo")

dev.new(width=7, height=7)
curve(1/(1+exp(-(b1*x + b0))), from=150, to=200, xlab="height",
      ylab="p(\"> Median\") bzw. Gruppe", lwd=2,
      main="Vorhersage der logistischen Regression", cex.lab=1.4)
abline(h=0.5, v=thresh, col="gray")
points(height[!wrong], jitter(as.numeric(wFac[!wrong] == "hi"), 0.1),
       col="black", pch=16)
points(height[ wrong], jitter(as.numeric(wFac[ wrong] == "hi"), 0.1), col="red",
       pch=16)
text(150, 0.95, adj=0, label="Daten ber dem Median")
text(200, 0.05, adj=1, label="Daten unter dem Median")
legend(x="right", c("Vorhersage p(\"> Median\")", "Trennwert fr p=0.5",
       "tatschliche Gruppe", "falsche Vorhersage"),
       col=c("black", "gray", "black", "red"), pch=c(NA, NA, 1, 16),
       lwd=c(2, 2, NA, NA), lty=c(1, 1, NA, NA))

summary(glmRes)

glm0 <- glm(wFac ~ 1, family=binomial(link="logit"), data=regDf)
anova(glm0, glmRes, test="Chisq")

statChisq <- glmRes$null.deviance - glmRes$deviance
dfChisq   <- glmRes$df.null       - glmRes$df.residual
(pVal     <- 1-pchisq(statChisq, dfChisq))

