#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 22.07.10
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Um alle Teile des Skripts ausfhren zu knnen, mssen die Pakete
## ICSNP, mvtnorm
## samt Abhngigkeiten installiert sein, vgl. Abschnitt 1.2.7:
##
## install.packages(c("ICSNP", "mvtnorm"), repos="http://ftp5.gwdg.de/pub/misc/cran/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

####*---------------------------------------------------------------------------
##### 9 Multivariate Verfahren
#####---------------------------------------------------------------------------

##### 9.1 Multivariate multiple Regression
nSubj  <- 100
height <- rnorm(nSubj, 175, 7)
age    <- rnorm(nSubj, 30, 8)
sport  <- abs(rnorm(nSubj, 60, 30))
weight <- 0.5*height - 0.3*age - 0.4*sport + 10 + rnorm(nSubj, 0, 3)
health <- -0.3*age + 0.6*sport + rnorm(nSubj, 4)
## fr Daten des Buches diese Datei laden:
# load("data_mvMultRegr.Rdata")
Y      <- cbind(weight, health)

(model <- lm(Y ~ height + age + sport))
lm(weight ~ height + age + sport)
lm(health ~ height + age + sport)

X   <- cbind(height, age, sport)
Sx  <- cov(X)
Sxy <- cov(X, Y)
(B  <- solve(Sx, Sxy))
(a  <- as.vector(colMeans(Y) - t(B) %*% colMeans(X)))

fitMat <- sweep(X %*% B, 2, a, "+")
all.equal(c(fitMat), c(fitted(model)))

#####---------------------------------------------------------------------------
##### 9.2 Hauptkomponentenanalyse
sigma <- matrix(c(4, 2, 2, 3), ncol=2)
mu    <- c(1, 2)
nSubj <- 500
library(mvtnorm)
X     <- rmvnorm(n=nSubj, mean=mu, sigma=sigma)
## fr Daten des Buches diese Datei laden:
# load("data_pca.Rdata")
(pca  <- prcomp(X))

coeff <- pca[[2]]
t(coeff) %*% coeff

ctr    <- colMeans(X)
eig    <- eigen(cov(X))
eigVal <- eig$values
eigVec <- eig$vectors
eigScl <- sweep(eigVec, 2, sqrt(eigVal), "*")
xMat   <- rbind(ctr[1] - eigScl[1, ], ctr[1])
yMat   <- rbind(ctr[2] - eigScl[2, ], ctr[2])

p1 <- ctr + eigScl[ , 1]
p2 <- ctr + eigScl[ , 2]
b1 <- (p1[2]-ctr[2]) / (p1[1]-ctr[1])
a1 <- ctr[2] - b1*ctr[1]
b2 <- (p2[2]-ctr[2]) / (p2[1]-ctr[1])
a2 <- ctr[2] - b2*ctr[1]

dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), cex.lab=1.2, cex.main=1.2)
plot(X[ , 1], X[ , 2], xlab="x", ylab="y", main="Datenwolke und Hauptkomponenten")
abline(a=a1, b=b1)
abline(a=a2, b=b2)
matlines(xMat, yMat, lty=1, lwd=6, col="blue")
points(ctr[1], ctr[2], pch=16, col="red", cex=3)
legend(x="topleft", legend=c("Daten", "Achsen HK", "Streuungen HK", "Zentroid"),
       pch=c(1, NA, NA, 16), lty=c(NA, 1, 1, NA),
       col=c("black", "black", "blue", "red"))

sqrt(eig$values)
eig$vectors

pc    <- X %*% coeff
(pcSd <- apply(pc, 2, sd))

summary(pca)
pca[[1]]^2 / sum(diag(cov(X)))

B    <- sweep(coeff, 2, pca[[1]], "*")
Hs   <- scale(pc)
mat  <- t(B %*% t(Hs))
repr <- sweep(mat, 2, ctr, "+")
all.equal(X, repr)
sum((X-repr)^2)

mat1  <- t(B[ , 1] %*% t(Hs[ , 1]))
repr1 <- sweep(mat1, 2, ctr, "+")
sum((X-repr1)^2)

plot(X[ , 1], X[ , 2], xlab="x", ylab="y", pch=3,
     main="Originaldaten und Approximation")
abline(a=a1, b=b1)
abline(a=a2, b=b2)
points(repr1, pch=1, lwd=2, col=rgb(0, 1, 0, 0.2), cex=2)
points(ctr[1], ctr[2], pch=16, col="red", cex=3)
legend(x="topleft", legend=c("Daten", "Achsen HK", "Zentroid", "Approximation"),
       pch=c(3, NA, 16, 1), lty=c(NA, 1, NA, NA),
       col=c("black", "black", "red", "green"))
par(op)

B %*% t(B)
cov(X)
B[ , 1] %*% t(B[ , 1])

(pcaPrin <- princomp(X))
pcaPrin$loadings

covMat <- cov.wt(pc, method="ML")$cov
sqrt(diag(covMat))

#####---------------------------------------------------------------------------
##### 9.3 Faktorenanalyse
nSubj   <- 200
nVar    <- 6
nFac    <- 2
(lambda <- matrix(c(0.7,-0.4, 0.8,0, -0.2,0.9, -0.3,0.4, 0.3,0.7, -0.8,0.1),
                  nrow=nVar, ncol=nFac, byrow=TRUE))
Kf      <- diag(nFac)
mu      <- c(5, 15)
library(mvtnorm)
f       <- t(rmvnorm(nSubj, mean=mu, sigma=Kf))
e       <- rnorm(nSubj, 0, 10)
X       <- t(lambda %*% f + e)

## fr Daten des Buches diese Datei laden:
# load("data_factanal.Rdata")
(fa <- factanal(X, factors=2, scores="regression"))

colSums((fa$loadings)^2)
colSums((fa$loadings)^2) / sum(diag(cor(X)))

Rr    <- fa$loadings %*% t(fa$loadings)
eigRr <- eigen(Rr)
eigRr$values

1 - fa$uniqueness
rowSums((fa$loadings)^2)

dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), cex.main=1.2, cex.lab=1.2)
plot(fa$loadings, xlab="Faktor 1", ylab="Faktor 2", xlim=c(-1.1, 1.1),
     ylim=c(-1.1, 1.1), pch=20, asp=1, main="Faktorenanalyse: Ladungen und Faktoren")
abline(h=0, v=0)
arrows(0, 0, c(1, 0), c(0, 1), col="blue", lwd=2)
angles <- seq(0, 2*pi, length.out=200)
circ   <- cbind(cos(angles), sin(angles))
lines(circ)

Vx    <- cov(X)
KxEst <- fa$loadings %*% t(fa$loadings) + diag(fa$uniquenesses)
VxEst <- diag(sqrt(diag(Vx))) %*% KxEst %*% diag(sqrt(diag(Vx)))

ang   <- pi/3
G     <- matrix(c(cos(ang), -sin(ang), sin(ang), cos(ang)), nrow=2, byrow=TRUE)
(Lrot <- fa$loadings %*% G)
KxEstRot <- Lrot %*% t(Lrot) + diag(fa$uniquenesses)
all.equal(KxEst, KxEstRot)

arrows(0, 0, G[1, ], G[2, ], col="green", lwd=2)
segments(0, 0, -G[1, ], -G[2, ])
text(fa$loadings[ , 1] , fa$loadings[ , 2]+0.08, label=paste("Var", 1:nVar))
legend(x="bottomright", legend=c("Ladungen", "Faktoren", "rotierte Faktoren"),
       pch=c(20, NA, NA), lty=c(NA, 1, 1), col=c("black", "blue", "green"))

plot(eigen(cor(X))$values, type="b", ylab="Eigenwert", main="Scree-Plot")
abline(h=1)
par(op)

G     <- matrix(c(5/3, -1, -4/3, 8/5), nrow=2, byrow=TRUE)
KfRot <- t(G) %*% diag(nFac) %*% G
Lrot  <- fa$loadings %*% solve(t(G))
facStruct <- fa$loadings %*% diag(nFac) %*% G
KxEstRot  <- Lrot %*% KfRot %*% t(Lrot) + diag(fa$uniquenesses)
all.equal(KxEst, KxEstRot)

#####---------------------------------------------------------------------------
##### 9.4 Multidimensionale Skalierung
cities  <- c("Augsburg", "Berlin", "Dresden", "Hamburg", "Hannover",
             "Karlsruhe", "Kiel", "Mnchen", "Rostock", "Stuttgart")
nCities <- length(cities)
dstMat  <- matrix(numeric(nCities^2), nrow=nCities)
#              B   DD   HH    H   KA   KI    M  HRO    S
cityDst <- c(596, 467, 743, 599, 226, 838,  65, 782, 160, # AUG
                  194, 288, 286, 673, 353, 585, 231, 633, # B
                       477, 367, 550, 542, 465, 420, 510, # DD
                            157, 623,  96, 775, 187, 665, # HH
                                 480, 247, 632, 330, 512, # H
                                      723, 298, 805,  80, # KA
                                           872, 206, 752, # KI
                                                777, 220, # M
                                                     824) # HRO

dstMat[upper.tri(dstMat)] <- rev(cityDst)
dstMat <- t(dstMat[ , ncol(dstMat):1])[ , ncol(dstMat):1]
dstMat[lower.tri(dstMat)] <- t(dstMat)[lower.tri(dstMat)]
dimnames(dstMat) <- list(city=cities, city=cities)

(mds <- cmdscale(dstMat, k=2))

xLims <- range(mds[ , 1]) + c(0, 250)
dev.new(width=7, height=7)
plot(mds, xlim=xLims, xlab="West-Ost", ylab="Sd-Nord", pch=16,
     main="Anordnung der Stdte nach MDS", cex.lab=1.4)
text(mds[ , 1]+50, mds[ , 2], adj=0, label=cities)

#####---------------------------------------------------------------------------
##### 9.5 Hotellings T^2
##### 9.5.1
sigma <- matrix(c(16,-1, -1,9), byrow=TRUE, ncol=2)
mu11  <- c(-3, 2)
library(mvtnorm)
nSubj <- c(15, 20, 25)
Y11   <- round(rmvnorm(n=nSubj[1], mean=mu11, sigma=sigma))
muH0  <- c(-1, 2)

## fr Daten des Buches diese Datei laden:
# load("data_manova.Rdata")
library(ICSNP)
HotellingsT2(Y11, mu=muH0)

Y11ctr <- sweep(Y11, 2, muH0, "-")
(anRes <- anova(lm(Y11ctr ~ 1), test="Hotelling-Lawley"))

n     <- nSubj[1]
ctr   <- colMeans(Y11)
(T2   <- n * (t(ctr-muH0) %*% solve(cov(Y11)) %*% (ctr-muH0)))
n * mahalanobis(ctr, muH0, cov(Y11))
(n-1) * anRes$"Hotelling-Lawley"[1]
P     <- 2
(T2f  <- ((n-P) / (P*(n-1))) * T2)
(pVal <- 1-pf(T2f, P, n-P))

##### 9.5.2
mu12 <- c(-2, 0)
Y12  <- round(rmvnorm(n=nSubj[2], mean=mu12, sigma=sigma))

## fr Daten des Buches diese Datei laden:
# load("data_manova.Rdata")
library(ICSNP)
HotellingsT2(Y11, Y12)

YH   <- rbind(Y11, Y12)
facH <- factor(rep(1:2, nSubj[1:2]))
anova(lm(YH ~ facH), test="Hotelling-Lawley")
(sumRes <- summary(manova(YH ~ facH), test="Hotelling-Lawley"))

n1    <- nSubj[1]
n2    <- nSubj[2]
ctr1  <- colMeans(Y11)
ctr2  <- colMeans(Y12)
S1    <- cov.wt(Y11, method="ML")$cov
S2    <- cov.wt(Y12, method="ML")$cov
Su    <- (1 / (n1+n2-2)) * (n1*S1 + n2*S2)
(T2   <- ((n1*n2) / (n1+n2)) * (t(ctr2-ctr1) %*% solve(Su) %*% (ctr2-ctr1)))
((n1*n2) / (n1+n2)) * mahalanobis(ctr1, ctr2, Su)
(n1+n2-2) * sumRes$stats[1, 2]
P     <- 2
(T2f  <- ((n1+n2-P-1) / ((n1+n2-2)*P)) * T2)
(pVal <- 1-pf(T2f, P, n1+n2-P-1))

#####---------------------------------------------------------------------------
##### 9.6 Multivariate Varianzanalyse
##### 9.6.1
mu13 <- c(-4, -1)
Y13  <- round(rmvnorm(n=nSubj[3], mean=mu13, sigma=sigma))
YM   <- rbind(Y11, Y12, Y13)
facM <- factor(rep(1:3, nSubj))
## fr Daten des Buches diese Datei laden:
# load("data_manova.Rdata")

manResO <- manova(YM ~ facM)
summary(manResO, test="Wilks")
summary(manResO, test="Roy")
summary(manResO, test="Pillai")
summary(manResO, test="Hotelling-Lawley")

X  <- cbind(as.numeric(facM == 1), as.numeric(facM == 2), as.numeric(facM == 3))
P  <- X %*% solve(t(X) %*% X) %*% t(X)
Pw <- diag(nrow(P)) - P

ones <- rep(1, nrow(X))
Pm   <- ones %*% solve(t(ones) %*% ones) %*% t(ones)
Pt   <- diag(nrow(Pm)) - Pm
Pb   <- P - Pm

TT <- t(YM) %*% Pt %*% YM
BB <- t(YM) %*% Pb %*% YM
WW <- t(YM) %*% Pw %*% YM

all.equal(TT, BB + WW)

(PB   <- sum( diag(solve(TT) %*% BB)))
(WL   <- det(WW) / det(TT))
(RLRt <- max(eigen(solve(TT) %*% BB)$values))
(RLRl <- max(eigen(solve(WW) %*% BB)$values))
RLRt / (1-RLRt)
(HL   <- sum( diag(solve(WW) %*% BB)))

##### 9.6.2
mu21  <- c(-2,  1)
mu22  <- c(-1, -1)
mu23  <- c(-5, -2)
Y21   <- round(rmvnorm(nSubj[1], mu21, sigma))
Y22   <- round(rmvnorm(nSubj[2], mu22, sigma))
Y23   <- round(rmvnorm(nSubj[3], mu23, sigma))
## fr Daten des Buches diese Datei laden:
# load("data_manova.Rdata")
YMt   <- rbind(YM, Y21, Y22, Y23)
facT1 <- rep(facM, 2)
facT2 <- factor(rep(1:2, each=sum(nSubj)))

manResT <- manova(YMt ~ facT1 + facT2)
summary(manResT, test="Pillai")
summary(manResT, test="Wilks")
summary(manResT, test="Roy")
summary(manResT, test="Hotelling-Lawley")

