#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 22.07.10
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Befehle, die eine Benutzerinteraktion notwendig machen oder
## nicht erwnschte Nebenwirkungen haben, sind auskommentiert.
##
## Um alle Teile des Skripts ausfhren zu knnen, mssen die Pakete
## gplots, rgl, scatterplot3d
## samt Abhngigkeiten installiert sein, vgl. Abschnitt 1.2.7:
##
## install.packages(c("gplots", "rgl", "scatterplot3d"),
##                  repos="http://ftp5.gwdg.de/pub/misc/cran/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

####*---------------------------------------------------------------------------
##### 10 Diagramme erstellen
#####---------------------------------------------------------------------------

#####---------------------------------------------------------------------------
##### 10.1 Graphik Devices
##### 10.1.1
# windows(); windows(); windows()
## plattformbergreifend (Windows, Linux, MacOS X):
dev.new(); dev.new(); dev.new()
dev.list()
dev.cur()
dev.set(3)
dev.set(dev.next())
dev.off()
graphics.off()

#####---------------------------------------------------------------------------
##### 10.1.2
pdf("pdf_test.pdf")
plot(1:10, rnorm(10))
dev.off()

plot(1:10, rnorm(10))
dev.copy(jpeg, filename="copied.jpg", quality=90)
graphics.off()

#####---------------------------------------------------------------------------
##### 10.2 Streu- und Liniendiagramme
##### 10.2.1
vec <- rnorm(1:10)
dev.new(width=10, height=7)
op <- par(mfrow=c(2, 3), cex.lab=1.4)
plot(vec, type="p", xlab=NA, main="type p", cex=1.5)
plot(vec, type="l", xlab=NA, main="type l", cex=1.5)
plot(vec, type="b", xlab=NA, main="type b", cex=1.5)
plot(vec, type="o", xlab=NA, main="type o", cex=1.5)
plot(vec, type="s", xlab=NA, main="type s", cex=1.5)
plot(vec, type="h", xlab=NA, main="type h", cex=1.5)
par(op)

vec1 <- sample(1:10, 200, replace=TRUE)
vec2 <- sample(1:10, 200, replace=TRUE)
dev.new(width=7, height=4)
op <- par(mfrow=c(1, 2), cex.main=0.9)
plot(vec2 ~ vec1, main="Punktwolke")
plot(jitter(vec2) ~ vec1, main="Punktwolke mit jitter")
par(op)

##### 10.2.2
vec <- rnorm(10)
dev.new(width=7, height=7)
plot(vec)
## identify(vec)

##### 10.2.3
vec <- seq(from=-2*pi, to=2*pi, length.out=50)
mat <- cbind(2*sin(vec), sin(vec-(pi/4)), 0.5*sin(vec-(pi/2)))
dev.new(width=7, height=7)
matplot(vec, mat, type="b", xlab=NA, ylab=NA, pch=1:3, main="Sinuskurven")

#####---------------------------------------------------------------------------
##### 10.3 Diagramme formatieren
##### 10.3.1
matX <- matrix(rep(1:6, 11),      ncol=11)
matY <- matrix(rep(0:10, each=6), ncol=11)

dev.new(width=7, height=7)
op <- par(mar=c(1, 1, 4, 2))
plot(0:6, seq(0, 10, length.out=length(0:6)), type="n", xlab=NA, ylab=NA,
     axes=FALSE, main="pch Datenpunkt-Symbole und lty Linientypen")
points(matX[1:26], matY[1:26], pch=0:25, bg="gray", cex=2)
matpoints(matX[ , 6:11], matY[ , 6:11], type="l", lty=6:1, lwd=2, col="black")
text(matX[1:26]-0.3, matY[1:26],    label=0:25)
text(rep(0.7, 6),    matY[1, 6:11], label=6:1)
text(0, 7.5, label="Linientypen fr lty", srt=90, cex=1.2)
text(0, 2.0, label="Symbole fr pch",     srt=90, cex=1.2)
par(op)

dev.new(width=7, height=4)
opv <- par(mfrow=c(1, 2), cex.main=0.9)
op  <- par(col="red", lwd=2, pch=16)
plot(rnorm(10), main="Rot, fett, gefllte Kreise")
par(op)
plot(rnorm(10), main="Standardformatierung")
par(opv)

##### 10.3.2
rgb(0, 1, 1)
rgb(t(col2rgb("red")/255))
hsv(0.1666, 1, 1)
gray(0.5)
## orgPal <- palette(rainbow(10))

##### 10.3.3

#####---------------------------------------------------------------------------
##### 10.4 Sulen- und Punktdiagramme
##### 10.4.1
dice  <- sample(1:6, 100, replace=TRUE)
## fr Daten des Buches diese Datei laden:
# load("data_barplot.Rdata")
(dTab <- table(dice))

dev.new(width=7, height=4)
op <- par(mfrow=c(1, 2), cex.main=0.9)
barplot(dTab, ylim=c(0, 30), xlab="Augenzahl", ylab="N", col="red",
        main="Absolute Hufigkeiten")
barplot(prop.table(table(dice)), ylim=c(0, 0.3), xlab="Augenzahl",
        ylab="relative Hufigkeit", col="blue", main="Relative Hufigkeiten")
par(op)

##### 10.4.2
## fr Daten des Buches diese Datei laden:
# load("data_barplot.Rdata")
roll1   <- dice[1:50]
roll2   <- dice[51:100]
rollAll <- rbind(table(roll1), table(roll2))
rownames(rollAll) <- c("first", "second"); rollAll

dev.new(width=12, height=4.5)
op <- par(mfrow=c(1, 3), cex.lab=1.5, cex.main=1.5)
barplot(rollAll, beside=FALSE, legend.text=TRUE, xlab="Augenzahl", ylab="N",
        main="Absolute Hufigkeiten in zwei Substichproben")
barplot(rollAll, ylim=c(0, 15), col=c("red", "green"), beside=TRUE,
        legend.text=TRUE, xlab="Augenzahl", ylab="N",
        main="Absolute Hufigkeiten in zwei Substichproben")

nSubj  <- 100
age    <- sample(18:45, nSubj, replace=TRUE)
drinks <- c("beer", "red wine", "white wine")
pref   <- factor(sample(drinks, nSubj, replace=TRUE))
xRange <- round(range(age), -1) + c(-10, 10)
lims   <- seq(xRange[1], xRange[2], by=10)
## fr Daten des Buches diese Datei laden:
# load("data_barplot.Rdata")
spineplot(x=age, y=pref, xlab="Altersstufe", ylab="Getrnk", breaks=lims,
          main="Bevorzugte Getrnke pro Altersstufe")
par(op)

##### 10.4.3
nSubj   <- 5
group1  <- rnorm(nSubj, 20, 2)
group2  <- rnorm(nSubj, 25, 2)
DV      <- c(group1, group2)
IV      <- factor(rep(1:2, each=nSubj))
groupMs <- tapply(DV, IV, mean)

dev.new(width=7, height=7)
dotchart(DV, gdata=groupMs, pch=20, color=rep(c("red", "blue"), each=nSubj),
         gcolor="black", labels=rep(LETTERS[1:nSubj], 2), groups=IV, xlab="AV",
         ylab="Gruppen", main="Individuelle Ergebnisse und Mittelwerte aus 2 Gruppen",
         cex=1.3, cex.lab=1.4, cex.main=1)

#####---------------------------------------------------------------------------
##### 10.5 Elemente einem bestehenden Diagramm hinzufgen
##### 10.5.1
dev.new(width=7, height=7)
plot(rnorm(10))
## locator(n=5)

##### 10.5.2
xA <- seq(-15, 15, length.out=200)
yA <- sin(xA) / xA
dev.new(width=7, height=7)
plot(xA, yA, type="l", xlab="x", ylab="sinc(x)",
     main="Punkte und Linien einfgen", lwd=1.6, cex.lab=1.4)

xB <- seq(-15, 15, length.out=30)
yB <- sin(xB) / xB
points(xB, yB, col="red", pch=20)

##### 10.5.3
yC <- sin(pi * xA) / (pi * xA)
lines(xA, yC, col="blue", type="l", lwd=1.6)
abline(h=0, v=0, col="green", lwd=1.6)

height <- rnorm(20, 175, 7)
weight <- 0.5*height + 10 + rnorm(20, 0, 4)
model  <- lm(weight ~ height)
pred   <- predict(model)
## fr Daten des Buches diese Datei laden:
# load("data_addGridArrows.Rdata")

dev.new(width=7, height=7)
plot(weight ~ height, asp=1, lwd=2, col="blue", pch=16,
     main="Gitter, Segmente und Pfeile einfgen", cex.lab=1.4)
abline(model, lwd=2)
grid()
segments(x0=height, y0=pred, x1=height, y1=weight, col="gray")
arrows(x0=c(height[1]-3, height[3]),   y0=c(weight[1], weight[3]+3),
       x1=c(height[1]-0.5, height[3]), y1=c(weight[1], weight[3]+0.5),
       col="red", lwd=2)
arrows(x0=height[4]+0.1*(height[8]-height[4]), y0=weight[4]+0.1*(weight[8]-weight[4]),
       x1=height[4]+0.9*(height[8]-height[4]), y1=weight[4]+0.9*(weight[8]-weight[4]),
       col="red", lwd=2, code=3)

##### 10.5.4
n      <- 7
len    <- 1/n
colsR  <- rep(seq(0.9, 0.2, length.out=n), each=n)
colsG  <- rep(seq(0.9, 0.2, length.out=n), n)
cols   <- rgb(colsR, colsG, 0)
## als Graustufen
## grays  <- rep(1:n, n) - rep(0:(n-1), each=n)
## grays  <- ((grays-min(grays)) / (max(grays)-min(grays)) * 0.8) + 0.1
## cols   <- gray(grays)
xLeft  <- rep(seq(0,   1-len, by=len), n)
yBot   <- rep(seq(0,   1-len, by=len), each=n)
xRight <- rep(seq(len, 1,     by=len), n)
yTop   <- rep(seq(len, 1,     by=len), each=n)

dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), oma=c(1, 1, 1, 1), mar=c(0, 0, 1, 0))
plot(c(0, 1), c(0, 1), axes=FALSE, type="n", asp=1, main="Farbverlauf",
     cex.main=1.2)
rect(xLeft, yBot, xRight, yTop, border=NA, col=cols)
idx   <- c(10, 27)
xText <- xLeft[idx] + (xRight[idx]-xLeft[idx])/2
yText <- yBot[idx]  + (yTop[idx]  - yBot[idx])/2
text(xText, yText, label=cols[idx])
shuffled <- sample(seq(along=cols), length(cols), replace=FALSE)
idxS     <- c(which(shuffled == idx[1]), which(shuffled == idx[2]))
plot(c(0, 1), c(0, 1), axes=FALSE, type="n", asp=1,
     main="Dieselben Farben zufllig angeordnet", cex.main=1.4)
rect(xLeft, yBot, xRight, yTop, border=NA, col=cols[shuffled])
xTextS   <- xLeft[idxS] + (xRight[idxS]-xLeft[idxS])/2
yTextS   <- yBot[idxS]  + (yTop[idxS]  - yBot[idxS])/2
text(xTextS, yTextS, label=cols[idx])
par(op)

mu    <- 0
sigma <- 3
xLims <- c(mu-4*sigma, mu+4*sigma)
myX   <- seq(xLims[1], xLims[2], length.out=100)
myY   <- dnorm(myX, mu, sigma)
selX  <- seq(mu-sigma, mu+sigma, length.out=100)
selY  <- dnorm(selX, mu, sigma)
cdf   <- pnorm(myX, mu, sigma)

dev.new(width=7, height=7)
op <- par(mar=c(5, 4, 4, 5), cex.lab=1.4)
plot(myX, myY, type="n", xlim=xLims-c(-2, 2), xlab=NA, ylab=NA,
     main="Dichtefunktion und Verteilungsfunktion N(0, 3)")
box(which="plot", col="gray", lwd=2)
polygon(c(selX, rev(selX)), c(selY, rep(-1, length(selX))), border=NA, col="lightgray")
points(myX, myY, type="l", lwd=2)
par(new=TRUE)
plot(myX, cdf, xlim=xLims-c(-2, 2), type="l", lwd=2, col="blue", xlab="x",
     ylab=NA, axes=FALSE)
axis(side=4, at=seq(0, 1, by=0.1), col="blue")
segments(x0=c(mu-sigma, mu, mu+sigma), y0=c(-1, -1, -1), x1=c(mu-sigma, mu, mu+sigma),
         y1=c(pnorm(mu-sigma, mu, sigma), pnorm(mu, mu, sigma), pnorm(mu+sigma, mu, sigma)),
         lwd=2, col=c("darkgreen", "red", "darkgreen"), lty=2)
segments(x0=c(mu-sigma, mu, mu+sigma), y0=c(pnorm(mu-sigma, mu, sigma), pnorm(mu, mu, sigma), pnorm(mu+sigma, mu, sigma)),
         x1=xLims[2]+10, y1=c(pnorm(mu-sigma, mu, sigma), pnorm(mu, mu, sigma), pnorm(mu+sigma, mu, sigma)),
         lwd=2, col=c("darkgreen", "red", "darkgreen"), lty=2)
arrows(x0=c(mu-sigma+0.2, mu+sigma-0.2), y0=-0.02, x1=c(mu-0.2, mu+0.2), y1=-0.02,
       code=3, angle=90, length=0.05, lwd=2, col="darkgreen")
mtext(text="F(x)", side=4, line=3, cex=1.4)
rect(-8.5, 0.92, -5.5, 1.0, col="lightgray", border=NA)
text(-7.2, 0.9, labels="Wahrscheinlichkeit")
text(-7.1, 0.86, expression(des~~Intervalls~~group("[", list(-sigma, sigma), "]")))
text(mu-sigma/2, 0,    expression(sigma), col="darkgreen", cex=1.2)
text(mu+sigma/2, 0,    expression(sigma), col="darkgreen", cex=1.2)
text(mu+0.5,     0.02, expression(mu),    col="red",       cex=1.2)
par(op)

##### 10.5.5
mu    <- 0
sigma <- 2
dev.new(width=7, height=7)
curve(dnorm, from=-7, to=7, col="blue", lwd=2, cex.lab=1.4)
curve((1/(sigma*sqrt(2*pi))) * exp(-0.5*(((x-mu)/sigma)^2)), add=TRUE, lwd=2, lty=2)

##### 10.5.6
title(main="zwei Normalverteilungskurven", sub="Untertitel")
legend(x="topleft", legend=c("N(0, 1)", "N(0, 2)"), col=c("blue", "black"), lty=c(1, 2))
text(x=3.5,  y=0.35, label="Standardnormalverteilung\nN(0, 1)")
text(x=-3.5, y=0.1 , label="N(0, 2)")
mtext(text="Wahrscheinlichkeitsdichte", side=3)
text(-4, 0.3, expression(frac(1, sigma*sqrt(2*pi)) ~~ exp*bgroup("(", -frac(1, 2) ~~ bgroup("(", frac(x-mu, sigma), ")")^2, ")")))

##### 10.5.7
vec <- seq(from=-2*pi, to=2*pi, length.out=200)
mat <- cbind(sin(vec), cos(vec), tan(vec))
mat <- ifelse(abs(mat) > 2, NA, mat)

dev.new(width=7, height=7)
matplot(vec, mat, lwd=2, col=c(12, 14, 17), type="l", lty=1, xaxt="n",
        xlab=NA, ylab=NA, main="Trigonometrische Funktionen")
xTicks  <- seq(from=-2*pi, to=2*pi, by=pi/2)
xLabels <- c("-2*pi", "-3*pi/2", "-pi", "-pi/2", "0", "pi/2", "pi", "3*pi/2", "2*pi")
axis(side=1, at=xTicks, labels=xLabels)
abline(h=c(-1, 0, 1), v=seq(from=-3*pi/2, to=3*pi/2, by=pi/2), col="gray", lty=3)
abline(h=0, v=0, lwd=2)
legend(x="bottomleft", legend=c("sin(x)", "cos(x)", "tan(x)"), lty=1, col=c(12, 14, 17))

##### 10.5.8 Fehlerbalken
##### 10.5.8.1
nSubj    <- 20
DV       <- c(rnorm(nSubj, 30, 6), rnorm(nSubj, 20, 6), rnorm(nSubj, 25, 6), 
              rnorm(nSubj, 15, 6))
IV       <- factor(rep(1:4, nSubj))
groupNs  <- tapply(DV, IV, length)
groupMs  <- tapply(DV, IV, mean)
groupSds <- tapply(DV, IV, sd)

dev.new(width=12, height=4.5)
op <- par(mfrow=c(1, 3), cex.lab=1.5, cex.main=1.5)
barsX    <- barplot(height=groupMs, ylim=c(0, 40), xaxt="n", xlab="Gruppe",
                    ylab="Mittelwert", main="Mittelwerte und Konfidenzintervalle")
axis(side=1, at=1:4, labels=LETTERS[1:4])
ciWidths <- qt(0.975, df=groupNs-1) * groupSds / sqrt(groupNs)
limHi    <- groupMs + ciWidths
limLo    <- groupMs - ciWidths
xWidth   <- 0.1
arrows(x0=barsX[1:2], y0=limLo[1:2], x1=barsX[1:2], y1=limHi[1:2], code=3,
       angle=90, length=xWidth, col="blue")
segments(x0=barsX[3:4],        y0=limLo[3:4],
         x1=barsX[3:4],        y1=limHi[3:4], col="red")
segments(x0=barsX[3:4]+xWidth, y0=limLo[3:4],
         x1=barsX[3:4]-xWidth, y1=limLo[3:4], col="red")
segments(x0=barsX[3:4]+xWidth, y0=limHi[3:4],
         x1=barsX[3:4]-xWidth, y1=limHi[3:4], col="red")

##### 10.5.8.2
library(gplots)
stripchart(DV ~ IV, method="jitter", xlab="Gruppe",
           main="Rohdaten und Konfidenzintervalle", xaxt="n",
           col="darkgray", ylim=c(0, 40), pch=16, vert=TRUE)
plotCI(x=groupMs, uiw=ciWidths, col="blue", lwd=2, pch=19, add=TRUE)
axis(side=1, at=1:4, labels=LETTERS[1:4])

means1 <- c(2, 3, 6, 3, 5)
sds1   <- c(1.7, 1.8, 1.7, 1.9, 1.8)
means2 <- c(4, 3, 2, 1, 3)
sds2   <- c(1.4, 1.7, 1.7, 1.3, 1.5)
xOff   <- 0.1
plotCI(y=c(means1, means2), x=c((1:5)-xOff, (1:5)+xOff), uiw=c(sds1, sds2),
       xlab="Faktor A", ylab="Mittelwert", ylim=c(0, 8),
       main="Mittelwerte und CIs im 5x2 Design", col=rep(c("blue", "red"), each=5),
       lty=rep(c(1, 2), each=5), pch=19, type="p")
legend(x="topleft", legend=c("B-1", "B-2"), pch=c(19, 19), col=c("blue", "red"))
par(op)

#####---------------------------------------------------------------------------
##### 10.6 Verteilungsdiagramme
##### 10.6.1
height <- rnorm(100, 175, 7)
dev.new(width=12, height=4.5)
op <- par(mfrow=c(1, 3), cex.main=1.5, cex.lab=1.5)
hist(height, xlab="height [cm]", ylab="N")
fromTo <- c(round(min(height), -1)-10, round(max(height), -1)+10)
limits <- seq(from=fromTo[1], to=fromTo[2], 5)
hist(height, freq=FALSE, xlim=fromTo, xlab="height [cm]",
     ylab="relative Hufigkeit", breaks=limits,
     main="Histogramm und Normalverteilung")
rug(jitter(height))
xNormal <- seq(from=fromTo[1], to=fromTo[2], length.out=50)
yNormal <- dnorm(xNormal, mean=mean(height), sd=sd(height))
lines(xNormal, yNormal, lwd=2, col="blue")

hist(height, freq=FALSE, xlim=fromTo, xlab="height [cm]",
     main="Histogramm und Schtzung der Dichte")
lines(density(height), lwd=2, col="blue")
rug(jitter(height))
par(op)

##### 10.6.2
stem(rnorm(100, 175, 7))

##### 10.6.3
nSubj   <- 40
nGroups <- 3
DV      <- rnorm(nGroups*nSubj, mean=100, sd=15)
IV      <- factor(rep(c("Control", "Group A", "Group B"), each=nSubj))
dev.new(width=5, height=5)
boxplot(DV ~ IV, ylab="Score", col=c("red", "blue", "green"),
        main="Boxplots der Scores in 3 Gruppen", cex.lab=1.2)

##### 10.6.4
nRolls  <- 25
nGroups <- 4
dice    <- sample(1:6, nGroups*nRolls, replace=TRUE)
group   <- factor(rep(1:nGroups, each=nRolls))
## fr Daten des Buches diese Datei laden:
# load("data_stripchart.Rdata")

dev.new(width=7, height=4)
op <- par(mfrow=c(1, 2), cex.main=0.9)
stripchart(dice ~ group, xlab="Augenzahl", ylab="Gruppe", pch=1,  col="blue",
           main="Wrfelwrfe - 4 Gruppen", sub="jitter-Methode", method="jitter")
stripchart(dice ~ group, xlab="Augenzahl", ylab="Gruppe", pch=16, col="red",
           main="Wrfelwrfe - 4 Gruppen", sub="stack-Methode",  method="stack")
par(op)

##### 10.6.5
vec1 <- rnorm(200)
vec2 <- rf(200, df1=3, df2=15)

dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), cex.main=1.2, cex.lab=1.2)
qqplot(vec1, vec2, xlab="Quantile N(0, 1)", ylab="Quantile F(3, 15)",
       main="Vergleich der Quantile von N(0, 1) und F(3, 15)-Verteilung")

height <- rnorm(100, 175, 7)
qqnorm(height)
qqline(height, col="red", lwd=2)
par(op)

##### 10.6.6
vec    <- round(rnorm(10), 1)
myStep <- ecdf(vec)
dev.new(width=7, height=7)
plot(myStep, main="Empirische kumulative Hufigkeitsverteilung", cex.lab=1.4)
curve(pnorm, add=TRUE, col="gray", lwd=2)

##### 10.6.7
dice <- sample(1:6, 100, replace=TRUE)
dTab <- table(dice)

dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), mar=c(1, 1, 4, 1))
pie(dTab, col=c("blue", "red", "yellow", "pink", "green", "orange"),
    main="Relative Hufigkeiten beim Wrfeln")

dTabFreq <- prop.table(dTab)
textRad  <- 0.5
angles   <- dTabFreq * 2 * pi
csAngles <- cumsum(angles)
csAngles <- csAngles - angles/2
textX    <- textRad * cos(csAngles)
textY    <- textRad * sin(csAngles)
text(x=textX, y=textY, label=dTabFreq)

pie(rep(1, 1000), labels="", border=NA, col=rainbow(1000),
    main="Spektrum als Farbkreis")
par(op)

##### 10.6.8
nSubj <- 200
x     <- rnorm(nSubj, 100, 15)
y     <- 0.5*x + rnorm(nSubj, 0, 10)
group <- factor(rep(LETTERS[1:2], each=nSubj/2))
## fr Daten des Buches diese Datei laden:
# load("data_distr2var.Rdata")

dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), cex.main=1.2, cex.lab=1.2)
plot(x, y, pch=c(4, 16)[unclass(group)], lwd=2,
     col=c("black", "darkgray")[unclass(group)],
     main="Gemeinsame Verteilung getrennt nach Gruppen")
legend(x="topleft", legend=c("Gruppe A", "Gruppe B"), pch=c(4, 16),
       col=c("black", "darkgray"))

mat    <- cbind(x, y)
ctr    <- colMeans(mat)
eigVal <- eigen(cov(mat))$values
eigVec <- eigen(cov(mat))$vectors
eigScl <- sweep(eigVec, 2, sqrt(eigVal), "*")
xMat   <- rbind(ctr[1] + eigScl[1, ], ctr[1] - eigScl[1, ])
yMat   <- rbind(ctr[2] + eigScl[2, ], ctr[2] - eigScl[2, ])

angles  <- seq(0, 2*pi, length.out=200)
ellBase <- cbind(sqrt(eigVal[1])*cos(angles), sqrt(eigVal[2])*sin(angles))
ellRot  <- eigVec %*% t(ellBase)

plot(mat[ , 1], mat[ , 2], xlab="x", ylab="y",
     main="Gemeinsame Verteilung zweier Variablen")
lines((ellRot+ctr)[1, ], (ellRot+ctr)[2, ], lwd=2, col="blue")
matlines(xMat, yMat, lty=1, lwd=2, col="green")
points(ctr[1], ctr[2], pch=4, col="red", lwd=3)

hullIdx <- chull(x, y)
polygon(x[hullIdx], y[hullIdx])
legend(x="bottomright", legend=c("Datenpunkte", "Zentroid", "Streuungsellipse",
       "Hauptachsen", "Konvexe Hlle"), pch=c(1, 4, NA, NA, NA),
       lty=c(NA, NA, 1, 1, 1), col=c("black", "red", "blue", "green", "black"))
par(op)

#####---------------------------------------------------------------------------
##### 10.7 Datenpunkte interpolieren
##### 10.7.1
xOne     <- 1:9
yOne     <- rnorm(9)
ptsLin   <- approx(xOne, yOne, method="linear",   n=30)
ptsConst <- approx(xOne, yOne, method="constant", n=30)
## fr Daten des Buches diese Datei laden:
# load("data_splines.Rdata")

dev.new(width=12, height=4.5)
op <- par(mfrow=c(1, 3), cex.lab=1.5, cex.main=1.5)
plot(xOne, yOne, xlab=NA, ylab=NA, pch=19, main="Datenpunkte interpolieren",
     cex=1.5)
points(ptsLin,   pch=16, col="red",  lwd=1.5)
points(ptsConst, pch=22, col="blue", lwd=1.5)
legend(x="bottomleft", c("Daten", "linear", "konstant"), pch=c(19, 16, 22),
       col=c("black", "red", "blue"))

xTwo  <- rnorm(100)
yTwo  <- 0.4 * xTwo + rnorm(100, 0, 1)

## fr Daten des Buches diese Datei laden:
# load("data_interp.Rdata")
ptsL1 <- loess.smooth(xTwo, yTwo, span=1/3)
ptsL2 <- loess.smooth(xTwo, yTwo, span=2/3)

plot(xTwo, yTwo, xlab=NA, ylab=NA, pch=16, main="Geglttetes Streudiagramm")
points(ptsL1, lwd=2, col="red",  type="l")
points(ptsL2, lwd=2, col="blue", type="l")

##### 10.7.2
## fr Daten des Buches diese Datei laden:
# load("data_interp.Rdata")
ord <- order(xTwo)
idx <- seq(8, 88, by=20)
xspline(xTwo[ord][idx], yTwo[ord][idx], c(1, -1, -1, 1, 1),
        border="darkgreen", lwd=2, open=FALSE)
legend(x="topleft", c("Daten", "Loess span 1/3", "Loess span 2/3", "xspline"),
       pch=c(19, NA, NA, NA), lty=c(NA, 1, 1, 1),
       col=c("black", "red", "blue", "darkgreen"))

plot(xOne, yOne, xlab=NA, ylab=NA, pch=19, main="Splines", cex=1.5)
ptsSpline <- spline(xOne, yOne, n=201)
smSpline1 <- smooth.spline(xOne, yOne, spar=0.25)
smSpline2 <- smooth.spline(xOne, yOne, spar=0.35)
smSpline3 <- smooth.spline(xOne, yOne, spar=0.45)
ptsX      <- seq(1, 9, length.out=201)
ptsSmSpl1 <- predict(smSpline1, ptsX)
ptsSmSpl2 <- predict(smSpline2, ptsX)
ptsSmSpl3 <- predict(smSpline3, ptsX)
lines(ptsSpline, col="darkgray", lwd=2)
matlines(x=ptsX, y=cbind(ptsSmSpl1$y, ptsSmSpl2$y, ptsSmSpl3$y),
         col=c("blue", "green", "orange"), lty=1, lwd=2)
legend(x="bottomleft", c("Daten", "Spline", "spar=0.25", "spar=0.35", "spar=0.45"),
       pch=c(19, NA, NA, NA, NA), lty=c(NA, 1, 1, 1, 1),
       col=c("black", "darkgray", "blue", "green", "orange"))
par(op)

#####---------------------------------------------------------------------------
##### 10.8 Diagramme fr dreidimensionale Daten
##### 10.8.1
muX    <- 1
muY    <- 3
sX     <- 1
sY     <- 1
rhoXY  <- 0.6
rangeX <- 2.5
rangeY <- 2.5
N      <- 50
myX    <- seq(from=muX-rangeX*sX, to=muX+rangeX*sX, length.out=N)
myY    <- seq(from=muY-rangeY*sY, to=muY+rangeY*sY, length.out=N)
aa     <-  1 / (2 * pi * sX * sY * sqrt(1-rhoXY^2))
bb     <- -1 / (2 *                    (1-rhoXY^2))
genC   <- function(x, y) { ((x-muX)^2 / sX^2) - (2*rhoXY*(x-muX)*(y-muY) / (sX*sY)) + ((y-muY)^2 / sY^2) }
cc     <- outer(myX, myY, "genC")
myZ    <- aa * exp(bb*cc)

dev.new(width=7, height=7)
contour(myX, myY, myZ, main="Hhenlinien fr 2D-NV Dichte", cex.main=1.6)
dev.new(width=7, height=7)
filled.contour(myX, myY, myZ, main="Farbige Hhenlinien fr 2D-NV Dichte",
               cex.main=1.6)

nSubj  <- 10
age    <- rnorm(nSubj, 30, 8)
sport  <- abs(-0.25*age + rnorm(nSubj, 60, 40))
weight <- -0.3*age -0.4*sport + 100 + rnorm(nSubj, 0, 3)
wScale <- (weight-min(weight)) * (0.8 / abs(diff(range(weight)))) + 0.2
## fr Daten des Buches diese Datei laden:
# load("data_symbols.Rdata")

dev.new(width=7, height=7)
symbols(age, sport, circles=wScale, inch=0.6, fg=NULL, bg=rainbow(nSubj),
        main="Gewicht in Abhngigkeit von Alter und Sport", cex.lab=1.6, cex.main=1.6)

##### 10.8.2
## fr Daten des Buches diese Datei laden:
# load("data_persp.Rdata")
dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), cex.main=1.4)
persp(myX, myY, myZ, xlab="x", ylab="y", zlab="Dichte", theta=5, phi=35,
      main="Dichte einer 2D Normalverteilung")

library(scatterplot3d)
vecX <- rep(seq(-10, 10, length.out=10), 10)
vecY <- rep(seq(-10, 10, length.out=10), each=10)
vecZ <- vecX*vecY
sp3  <- scatterplot3d(vecX, vecY, vecZ, type="h", main="3D Scatterplot")
sp3$points3d(vecX, vecY, vecZ, col="red", pch=16)
par(op)

library(rgl)
plot3d(vecX, vecY, vecZ, col="blue", type="s")

##### 10.8.3
nSubj  <- 25
P      <- 2
Q      <- 2
IQ     <- rnorm(P*Q*nSubj, mean=100, sd=15)
height <- rnorm(P*Q*nSubj, 175, 7)
IV1    <- factor(rep(c("control", "treatment"), each=Q*nSubj))
IV2    <- factor(rep(c("f", "m"), P*nSubj))
myDf   <- data.frame(IV1, IV2, IQ, height)
dev.new(width=7, height=7)
coplot(IQ ~ height | IV1*IV2, data=myDf)

library(lattice)
dev.new(width=7, height=7)
histogram(IQ ~ height | IV1*IV2, data=myDf, main="Histogramme pro Gruppe")

##### 10.8.4
nSubj  <- 20
P      <- 2
group  <- rep(c("CG", "T"), each=nSubj/P)
age    <- sample(18:35, nSubj, replace=TRUE)
IQ     <- round(rnorm(nSubj, mean=rep(c(100, 115), each=nSubj/P), sd=15))
rating <- round(0.4*IQ - 30 + rnorm(nSubj, 0, 10), 1)
score  <- round(-0.3*IQ + 0.7*age + rnorm(nSubj, 0, 8), 1)
## fr Daten des Buches diese Datei laden:
# load("data_pairs.Rdata")
(mvDf  <- data.frame(group, age, IQ, rating, score))

dev.new(width=7, height=7)
pairs(mvDf[c("age", "IQ", "rating", "score")], main="Streudiagramm-Matrix",
      pch=16, col=c("red", "blue")[unclass(mvDf$group)])

myHist <- function(x, ...)    { par(new=TRUE); hist(x, ..., main="") }
myEll  <- function(x, y, ...) {
    xLL      <- tapply(x, mvDf$group, "[")
    yLL      <- tapply(y, mvDf$group, "[")
    matCG    <- cbind(xLL$CG, yLL$CG)
    matT     <- cbind(xLL$T, yLL$T)
    ctrCG    <- colMeans(matCG)
    ctrT     <- colMeans(matT)
    RRcg     <- chol(cov(matCG))
    RRt      <- chol(cov(matT))
    angles   <- seq(0, 2*pi, length.out=200)
    ellCG    <- cbind(cos(angles), sin(angles)) %*% RRcg
    ellT     <- cbind(cos(angles), sin(angles)) %*% RRt
    ellCGctr <- sweep(ellCG, 2, ctrCG, "+")
    ellTctr  <- sweep(ellT,  2, ctrT,  "+")
    
    points(ctrCG[1], ctrCG[2], col="red", pch=4, lwd=2)
    lines(ellCGctr, col="red")
    
    points(ctrT[1],  ctrT[2], col="blue", pch=4, lwd=2)
    lines(ellTctr, col="blue")
}

## fr Daten des Buches diese Datei laden:
# load("data_pairs.Rdata")
pairs(mvDf[c("age", "IQ", "rating", "score")], diag.panel=myHist,
      upper.panel=myEll, main="Streudiagramm-Matrix", pch=16,
      col=c("red", "blue")[unclass(mvDf$group)])

#####---------------------------------------------------------------------------
##### 10.9 Mehrere Diagramme in einem Graphik-Device darstellen
##### 10.9.1
(mat1 <- matrix(1:4, 2, 2))
dev.new(width=5, height=5)
layout(mat1)
op <- par(lwd=3, cex=3)
layout.show(4)
par(op)

dev.new(width=5, height=5)
layout(mat1, widths=c(1, 2), heights=c(1, 2))
op <- par(lwd=3, cex=3)
layout.show(4)
par(op)

dev.new(width=7, height=7)
layout(mat1, widths=c(1, 2), heights=c(1, 2))
barplot(table(round(rnorm(100))), horiz=TRUE, main="Balkendiagramm")
boxplot(rt(100, df=5), main="Box-Whisker Plot")
stripchart(sample(1:20, 40, replace=TRUE), method="stack", main="Stripchart")
pie(table(sample(1:6, 20, replace=TRUE)), main="Kreisdiagramm")

(mat2 <- matrix(c(1, 0, 1, 2), 2, 2))
dev.new(width=7, height=7)
layout(mat2)
stripchart(sample(1:20, 40, replace=TRUE), method="stack", main="Stripchart")
barplot(table(round(rnorm(100))), main="Sulendiagramm")

##### 10.9.2
dev.new(width=7, height=4)
op <- par(mfrow=c(1, 2))
boxplot(rt(100, df=5), xlab=NA, notch=TRUE, main="Box-Whisker Plot")
plot(rnorm(10), pch=16, xlab=NA, ylab=NA, main="Streudiagramm")
par(op)

resBinom <- rbinom(1000, size=10, prob=0.3)
facBinom <- factor(resBinom, levels=0:10)
tabBinom <- table(facBinom)

dev.new(width=7, height=7)
op <- par(fig=c(0, 1, 0.20, 1), cex.lab=1.4)
plot(tabBinom, type="h", bty="n", xlim=c(0, 10), xlab=NA, ylab="Hufigkeit",
     main="Ergebnisse von 1000*10 Bernoulli Experimenten (p=0.3)")
points(names(tabBinom), tabBinom, pch=16, col="red", cex=2)
par(fig=c(0, 1, 0, 0.35), bty="n", new=TRUE)
boxplot(resBinom, horizontal=TRUE, ylim=c(0, 10), notch=TRUE, col="gray",
        xlab="Anzahl der Erfolge")
par(op)

##### 10.9.3
dev.new(width=7, height=7)
splitMat <- rbind(c(0, 0.5, 0, 0.5), c(0.15, 0.85, 0.15, 0.85), c(0.5, 1, 0.5, 1))
split.screen(splitMat)
screen(1)
barplot(table(round(rnorm(100))), main="Sulendiagramm")
screen(2)
boxplot(sample(1:20, 100, replace=TRUE) ~ factor(rep(LETTERS[1:4], each=25)),
        col=rainbow(4), notch=TRUE, main="Box-Whisker Plot")
screen(3)
plot(sample(1:20, 40, replace=TRUE), pch=20, xlab=NA, ylab=NA,
     main="Streudiagramm")
close.screen(all.screens=TRUE)

