#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 22.07.10
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

####*---------------------------------------------------------------------------
##### 11 R als Programmiersprache
#####---------------------------------------------------------------------------

#####---------------------------------------------------------------------------
##### 11.1 Eigene Funktionen erstellen
sd

##### 11.1.1
##### 11.1.2
myPlot <- function(x, y, xLims=xRange) {
    xRange <- round(range(x), -1) + c(-10, 10)
    plot(x, y, xlim=xLims)
}

##### 11.1.3
##### 11.1.4
dnorm100.5   <- function(x) { dnorm(x, mean=100, sd=5)   }
dnorm100.7.5 <- function(x) { dnorm(x, mean=100, sd=7.5) }
dnorm100.10  <- function(x) { dnorm(x, mean=100, sd=10)  }
dnorm100.15  <- function(x) { dnorm(x, mean=100, sd=15)  }

dev.new(width=7, height=7)
curve(dnorm100.5,   from=50, to=150, col="green", lwd=2,
      main="Normalverteilungen", xlab=NA, ylab="Dichte", cex.lab=1.4)
curve(dnorm100.7.5, from=50, to=150, col="red",   lwd=2, add=TRUE)
curve(dnorm100.10,  from=50, to=150, col="blue",  lwd=2, add=TRUE)
curve(dnorm100.15,  from=50, to=150, col="black", lwd=2, add=TRUE)
legend(x="topleft", legend=c("N(100, 5)", "N(100, 7.5)", "N(100, 10)", "N(100, 15)"),
       lwd=2, col=c("green", "red", "blue", "black"))

c(1, 2, 3)[1]
(function(arg1, arg2) { arg1^2 + arg2^2 })(-3, 4)

mat <- matrix(round(rnorm(16, 100, 15)), 4, 4)
apply(mat, 2, function(x) { sqrt(sum(x^2)) })

##### 11.1.5
info            <- function(x) { UseMethod("info") }
info.numeric    <- function(x) { range(x)  }
info.matrix     <- function(x) { dim(x)    }
info.data.frame <- function(x) { names(x)  }
info.default    <- function(x) { length(x) }

vec  <- round(runif(12, 20, 100), 2)
char <- LETTERS[sample(1:26, 5)]
mat  <- matrix(vec, nrow=3)
myDf <- data.frame(mat)
names(myDf) <- LETTERS[1:ncol(myDf)]

info(vec)
info(mat)
info(myDf)
info(char)

#####---------------------------------------------------------------------------
##### 11.2 Kontrollstrukturen
##### 11.2.1
(x <- round(rnorm(1, 100, 15)))
if(x > 100) { cat("x is", x, "(greater than 100)\n") }

x <- round(rnorm(1, 100, 15))
if (x > 100) {
    cat("x is", x, "(greater than 100)\n")
} else {
    cat("x is", x, "(100 or less)\n")
}

(day <- sample(c("Mon", "Tue", "Wed"), 1))
if(day == "Mon") {
    print("The day is Monday")
} else {
    if(day == "Tue") {
        print("The day is Tuesday")
    } else {
        print("The day is Wednsday")
    }
}

(val <- sample(1:3, 1))
switch(val, print("val is 1"), print("val is 2"), print("val is 3"))
myCalc <- function(op, vals) {
    switch(op,
           plus  = vals[1] + vals[2],
           minus = vals[1] - vals[2],
           times = vals[1] * vals[2],
           vals[1] / vals[2])
}

(vals <- round(rnorm(2, 1, 10)))
myCalc("minus", vals)
myCalc("XX", vals)

##### 11.2.2
ABC <- c("Alfa", "Bravo", "Charlie", "Delta")
for(i in ABC) { print(i) }

src     <- log((rnorm(100000, 0, 1))^2)
alpha   <- 0.05
nTests  <- 1000
n       <- 20
sigVecT <- numeric(nTests)
sigVecV <- numeric(nTests)

for (i in seq(along=numeric(nTests))) {
    group1     <- sample(src, n, replace=FALSE)
    group2     <- sample(src, n, replace=FALSE)
    resT       <-   t.test(group1, group2)
    resV       <- var.test(group1, group2)
    sigVecT[i] <- as.logical(resT$p.value < alpha)
    sigVecV[i] <- as.logical(resV$p.value < alpha)
}

cat("Erwartete Anzahl signifikanter Tests:", alpha*nTests, "\n")
cat("Signifikante t-Tests:", sum(sigVecT), "\n")
cat("Signifikante F-Tests auf Varianzhomogenitt:", sum(sigVecV), "\n")

group1 <- replicate(nTests, sample(src, n, replace=FALSE))
group2 <- replicate(nTests, sample(src, n, replace=FALSE))
groups <- rbind(group1, group2)

estSigDiffs <- apply(groups, 2, function(x) {
               sqrt(1/n) * sqrt(var(x[1:n]) + var(x[(n+1):(2*n)])) } )

meanDiffs <- apply(groups, 2, function(x) {
             mean(x[1:n]) - mean(x[(n+1):(2*n)]) } )

tVals <- meanDiffs / estSigDiffs
pVals <- 1-pt(tVals, (2*n)-2)
sum(pVals < alpha)

myMod <- function(x, y) {
    if((x < 0) | (y < 0)) { return(invisible(NULL)) }
    while(abs(x) >= abs(y)) { x <- x-y }
    print(x)
}
myMod(37, 10)
myMod(5, 17)

for(i in 1:10) {
    if((i %% 2) != 0) { next  }
    if((i %% 8) == 0) { break }
    print(i)
}

i <- 0
repeat {
    i <- i+1
    if((i %% 4) == 0) { break }
    print(i)
}
